<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */
// +----------------------------------------------------------------------+
// | PHP version 4                                                        |
// +----------------------------------------------------------------------+
// | Copyright (c) 1997-2002 The PHP Group                                |
// +----------------------------------------------------------------------+
// | This source file is subject to version 2.0 of the PHP license,       |
// | that is bundled with this package in the file LICENSE, and is        |
// | available through the world-wide-web at                              |
// | http://www.php.net/license/2_02.txt.                                 |
// | If you did not receive a copy of the PHP license and are unable to   |
// | obtain it through the world-wide-web, please send a note to          |
// | license@php.net so we can mail you a copy immediately.               |
// +----------------------------------------------------------------------+
// | Author: Xavier Noguer <xnoguer@php.net>                              |
// +----------------------------------------------------------------------+
//
// $Id$

require_once('PEAR.php');
require_once('File/DICOM/Element.php');


define('FILE_DICOM_VR_TYPE_EXPLICIT_32_BITS', 0);
define('FILE_DICOM_VR_TYPE_EXPLICIT_16_BITS', 1);
define('FILE_DICOM_VR_TYPE_IMPLICIT', 2);

/**
* This class allows reading and modifying of DICOM files
*
* @author   Xavier Noguer <xnoguer@php.net>
* @package  File_DICOM
*/
class File_DICOM extends PEAR
{
    /**
    * DICOM dictionary.
    * @var array
    */
    var $dict;

    /**
    * Flag indicating if the current file is a DICM file or a NEMA file.
    * true => DICM, false => NEMA.
    *
    * @var bool
    */
    var $is_dicm;

    /**
    * Currently open file.
    * @var string
    */
    var $current_file;

    /**
    * Initial 0x80 bytes of last read file
    * @var string
    */
    var $_preamble_buff;

    /**
    * Definitions of fields of DICOM headers.
    * Andrew Crabb (ahc@jhu.edu), May 2002.
    *
    * ------------------------------------------------------------------------
    * NOT FOR MEDICAL USE.  This file is provided purely for experimental use.
    * ------------------------------------------------------------------------
    * @var array
    */

    var $dicom_fields = array(
array(0x0000,0x0000,'UL','1','GroupLength'),
array(0x0000,0x0001,'UL','1','CommandLengthToEnd'),
array(0x0000,0x0002,'UI','1','AffectedSOPClassUID'),
array(0x0000,0x0003,'UI','1','RequestedSOPClassUID'),
array(0x0000,0x0010,'CS','1','CommandRecognitionCode'),
array(0x0000,0x0100,'US','1','CommandField'),
array(0x0000,0x0110,'US','1','MessageID'),
array(0x0000,0x0120,'US','1','MessageIDBeingRespondedTo'),
array(0x0000,0x0200,'AE','1','Initiator'),
array(0x0000,0x0300,'AE','1','Receiver'),
array(0x0000,0x0400,'AE','1','FindLocation'),
array(0x0000,0x0600,'AE','1','MoveDestination'),
array(0x0000,0x0700,'US','1','Priority'),
array(0x0000,0x0800,'US','1','DataSetType'),
array(0x0000,0x0850,'US','1','NumberOfMatches'),
array(0x0000,0x0860,'US','1','ResponseSequenceNumber'),
array(0x0000,0x0900,'US','1','Status'),
array(0x0000,0x0901,'AT','1-n','OffendingElement'),
array(0x0000,0x0902,'LO','1','ErrorComment'),
array(0x0000,0x0903,'US','1','ErrorID'),
array(0x0000,0x0904,'OT','1-n','ErrorInformation'),
array(0x0000,0x1000,'UI','1','AffectedSOPInstanceUID'),
array(0x0000,0x1001,'UI','1','RequestedSOPInstanceUID'),
array(0x0000,0x1002,'US','1','EventTypeID'),
array(0x0000,0x1003,'OT','1-n','EventInformation'),
array(0x0000,0x1005,'AT','1-n','AttributeIdentifierList'),
array(0x0000,0x1007,'AT','1-n','ModificationList'),
array(0x0000,0x1008,'US','1','ActionTypeID'),
array(0x0000,0x1009,'OT','1-n','ActionInformation'),
array(0x0000,0x1013,'UI','1-n','SuccessfulSOPInstanceUIDList'),
array(0x0000,0x1014,'UI','1-n','FailedSOPInstanceUIDList'),
array(0x0000,0x1015,'UI','1-n','WarningSOPInstanceUIDList'),
array(0x0000,0x1020,'US','1','NumberOfRemainingSuboperations'),
array(0x0000,0x1021,'US','1','NumberOfCompletedSuboperations'),
array(0x0000,0x1022,'US','1','NumberOfFailedSuboperations'),
array(0x0000,0x1023,'US','1','NumberOfWarningSuboperations'),
array(0x0000,0x1030,'AE','1','MoveOriginatorApplicationEntityTitle'),
array(0x0000,0x1031,'US','1','MoveOriginatorMessageID'),
array(0x0000,0x4000,'AT','1','DialogReceiver'),
array(0x0000,0x4010,'AT','1','TerminalType'),
array(0x0000,0x5010,'SH','1','MessageSetID'),
array(0x0000,0x5020,'SH','1','EndMessageSet'),
array(0x0000,0x5110,'AT','1','DisplayFormat'),
array(0x0000,0x5120,'AT','1','PagePositionID'),
array(0x0000,0x5130,'CS','1','TextFormatID'),
array(0x0000,0x5140,'CS','1','NormalReverse'),
array(0x0000,0x5150,'CS','1','AddGrayScale'),
array(0x0000,0x5160,'CS','1','Borders'),
array(0x0000,0x5170,'IS','1','Copies'),
array(0x0000,0x5180,'CS','1','OldMagnificationType'),
array(0x0000,0x5190,'CS','1','Erase'),
array(0x0000,0x51A0,'CS','1','Print'),
array(0x0000,0x51B0,'US','1-n','Overlays'),
array(0x0002,0x0000,'UL','1','MetaElementGroupLength'),
array(0x0002,0x0001,'OB','1','FileMetaInformationVersion'),
array(0x0002,0x0002,'UI','1','MediaStorageSOPClassUID'),
array(0x0002,0x0003,'UI','1','MediaStorageSOPInstanceUID'),
array(0x0002,0x0010,'UI','1','TransferSyntaxUID'),
array(0x0002,0x0012,'UI','1','ImplementationClassUID'),
array(0x0002,0x0013,'SH','1','ImplementationVersionName'),
array(0x0002,0x0016,'AE','1','SourceApplicationEntityTitle'),
array(0x0002,0x0100,'UI','1','PrivateInformationCreatorUID'),
array(0x0002,0x0102,'OB','1','PrivateInformation'),
array(0x0004,0x0000,'UL','1','FileSetGroupLength'),
array(0x0004,0x1130,'CS','1','FileSetID'),
array(0x0004,0x1141,'CS','8','FileSetDescriptorFileID'),
array(0x0004,0x1142,'CS','1','FileSetCharacterSet'),
array(0x0004,0x1200,'UL','1','RootDirectoryFirstRecord'),
array(0x0004,0x1202,'UL','1','RootDirectoryLastRecord'),
array(0x0004,0x1212,'US','1','FileSetConsistencyFlag'),
array(0x0004,0x1220,'SQ','1','DirectoryRecordSequence'),
array(0x0004,0x1400,'UL','1','NextDirectoryRecordOffset'),
array(0x0004,0x1410,'US','1','RecordInUseFlag'),
array(0x0004,0x1420,'UL','1','LowerLevelDirectoryOffset'),
array(0x0004,0x1430,'CS','1','DirectoryRecordType'),
array(0x0004,0x1432,'UI','1','PrivateRecordUID'),
array(0x0004,0x1500,'CS','8','ReferencedFileID'),
array(0x0004,0x1504,'UL','1','DirectoryRecordOffset'),
array(0x0004,0x1510,'UI','1','ReferencedSOPClassUIDInFile'),
array(0x0004,0x1511,'UI','1','ReferencedSOPInstanceUIDInFile'),
array(0x0004,0x1512,'UI','1','ReferencedTransferSyntaxUIDInFile'),
array(0x0004,0x1600,'UL','1','NumberOfReferences'),
array(0x0008,0x0000,'UL','1','IdentifyingGroupLength'),
array(0x0008,0x0001,'UL','1','LengthToEnd'),
array(0x0008,0x0005,'CS','1','SpecificCharacterSet'),
array(0x0008,0x0008,'CS','1-n','ImageType'),
array(0x0008,0x000A,'US','1','SequenceItemNumber'),
array(0x0008,0x0010,'CS','1','RecognitionCode'),
array(0x0008,0x0012,'DA','1','InstanceCreationDate'),
array(0x0008,0x0013,'TM','1','InstanceCreationTime'),
array(0x0008,0x0014,'UI','1','InstanceCreatorUID'),
array(0x0008,0x0016,'UI','1','SOPClassUID'),
array(0x0008,0x0018,'UI','1','SOPInstanceUID'),
array(0x0008,0x0020,'DA','1','StudyDate'),
array(0x0008,0x0021,'DA','1','SeriesDate'),
array(0x0008,0x0022,'DA','1','AcquisitionDate'),
array(0x0008,0x0023,'DA','1','ImageDate'),
array(0x0008,0x0024,'DA','1','OverlayDate'),
array(0x0008,0x0025,'DA','1','CurveDate'),
array(0x0008,0x002A,'DT','1','AcquisitionDatetime'),
array(0x0008,0x0030,'TM','1','StudyTime'),
array(0x0008,0x0031,'TM','1','SeriesTime'),
array(0x0008,0x0032,'TM','1','AcquisitionTime'),
array(0x0008,0x0033,'TM','1','ImageTime'),
array(0x0008,0x0034,'TM','1','OverlayTime'),
array(0x0008,0x0035,'TM','1','CurveTime'),
array(0x0008,0x0040,'US','1','OldDataSetType'),
array(0x0008,0x0041,'LT','1','OldDataSetSubtype'),
array(0x0008,0x0042,'CS','1','NuclearMedicineSeriesType'),
array(0x0008,0x0050,'SH','1','AccessionNumber'),
array(0x0008,0x0052,'CS','1','QueryRetrieveLevel'),
array(0x0008,0x0054,'AE','1-n','RetrieveAETitle'),
array(0x0008,0x0058,'UI','1-n','DataSetFailedSOPInstanceUIDList'),
array(0x0008,0x0060,'CS','1','Modality'),
array(0x0008,0x0061,'CS','1-n','ModalitiesInStudy'),
array(0x0008,0x0064,'CS','1','ConversionType'),
array(0x0008,0x0068,'CS','1','PresentationIntentType'),
array(0x0008,0x0070,'LO','1','Manufacturer'),
array(0x0008,0x0080,'LO','1','InstitutionName'),
array(0x0008,0x0081,'ST','1','InstitutionAddress'),
array(0x0008,0x0082,'SQ','1','InstitutionCodeSequence'),
array(0x0008,0x0090,'PN','1','ReferringPhysicianName'),
array(0x0008,0x0092,'ST','1','ReferringPhysicianAddress'),
array(0x0008,0x0094,'SH','1-n','ReferringPhysicianTelephoneNumber'),
array(0x0008,0x0100,'SH','1','CodeValue'),
array(0x0008,0x0102,'SH','1','CodingSchemeDesignator'),
array(0x0008,0x0103,'SH','1','CodingSchemeVersion'),
array(0x0008,0x0104,'LO','1','CodeMeaning'),
array(0x0008,0x0105,'CS','1','MappingResource'),
array(0x0008,0x0106,'DT','1','ContextGroupVersion'),
array(0x0008,0x0107,'DT','1','ContextGroupLocalVersion'),
array(0x0008,0x010B,'CS','1','CodeSetExtensionFlag'),
array(0x0008,0x010C,'UI','1','PrivateCodingSchemeCreatorUID'),
array(0x0008,0x010D,'UI','1','CodeSetExtensionCreatorUID'),
array(0x0008,0x010F,'CS','1','ContextIdentifier'),
array(0x0008,0x0201,'SH','1','TimezoneOffsetFromUTC'),
array(0x0008,0x1000,'AE','1','NetworkID'),
array(0x0008,0x1010,'SH','1','StationName'),
array(0x0008,0x1030,'LO','1','StudyDescription'),
array(0x0008,0x1032,'SQ','1','ProcedureCodeSequence'),
array(0x0008,0x103E,'LO','1','SeriesDescription'),
array(0x0008,0x1040,'LO','1','InstitutionalDepartmentName'),
array(0x0008,0x1048,'PN','1-n','PhysicianOfRecord'),
array(0x0008,0x1050,'PN','1-n','PerformingPhysicianName'),
array(0x0008,0x1060,'PN','1-n','PhysicianReadingStudy'),
array(0x0008,0x1070,'PN','1-n','OperatorName'),
array(0x0008,0x1080,'LO','1-n','AdmittingDiagnosisDescription'),
array(0x0008,0x1084,'SQ','1','AdmittingDiagnosisCodeSequence'),
array(0x0008,0x1090,'LO','1','ManufacturerModelName'),
array(0x0008,0x1100,'SQ','1','ReferencedResultsSequence'),
array(0x0008,0x1110,'SQ','1','ReferencedStudySequence'),
array(0x0008,0x1111,'SQ','1','ReferencedStudyComponentSequence'),
array(0x0008,0x1115,'SQ','1','ReferencedSeriesSequence'),
array(0x0008,0x1120,'SQ','1','ReferencedPatientSequence'),
array(0x0008,0x1125,'SQ','1','ReferencedVisitSequence'),
array(0x0008,0x1130,'SQ','1','ReferencedOverlaySequence'),
array(0x0008,0x1140,'SQ','1','ReferencedImageSequence'),
array(0x0008,0x1145,'SQ','1','ReferencedCurveSequence'),
array(0x0008,0x114A,'SQ','1','ReferencedInstanceSequence'),
array(0x0008,0x114B,'LO','1','ReferenceDescription'),
array(0x0008,0x1150,'UI','1','ReferencedSOPClassUID'),
array(0x0008,0x1155,'UI','1','ReferencedSOPInstanceUID'),
array(0x0008,0x115A,'UI','1-n','SOPClassesSupported'),
array(0x0008,0x1160,'IS','1','ReferencedFrameNumber'),
array(0x0008,0x1195,'UI','1','TransactionUID'),
array(0x0008,0x1197,'US','1','FailureReason'),
array(0x0008,0x1198,'SQ','1','FailedSOPSequence'),
array(0x0008,0x1199,'SQ','1','ReferencedSOPSequence'),
array(0x0008,0x2110,'CS','1','LossyImageCompression'),
array(0x0008,0x2111,'ST','1','DerivationDescription'),
array(0x0008,0x2112,'SQ','1','SourceImageSequence'),
array(0x0008,0x2120,'SH','1','StageName'),
array(0x0008,0x2122,'IS','1','StageNumber'),
array(0x0008,0x2124,'IS','1','NumberOfStages'),
array(0x0008,0x2128,'IS','1','ViewNumber'),
array(0x0008,0x2129,'IS','1','NumberOfEventTimers'),
array(0x0008,0x212A,'IS','1','NumberOfViewsInStage'),
array(0x0008,0x2130,'DS','1-n','EventElapsedTime'),
array(0x0008,0x2132,'LO','1-n','EventTimerName'),
array(0x0008,0x2142,'IS','1','StartTrim'),
array(0x0008,0x2143,'IS','1','StopTrim'),
array(0x0008,0x2144,'IS','1','RecommendedDisplayFrameRate'),
array(0x0008,0x2200,'CS','1','TransducerPosition'),
array(0x0008,0x2204,'CS','1','TransducerOrientation'),
array(0x0008,0x2208,'CS','1','AnatomicStructure'),
array(0x0008,0x2218,'SQ','1','AnatomicRegionSequence'),
array(0x0008,0x2220,'SQ','1','AnatomicRegionModifierSequence'),
array(0x0008,0x2228,'SQ','1','PrimaryAnatomicStructureSequence'),
array(0x0008,0x2229,'SQ','1','AnatomicStructureSpaceOrRegionSequence'),
array(0x0008,0x2230,'SQ','1','PrimaryAnatomicStructureModifierSequence'),
array(0x0008,0x2240,'SQ','1','TransducerPositionSequence'),
array(0x0008,0x2242,'SQ','1','TransducerPositionModifierSequence'),
array(0x0008,0x2244,'SQ','1','TransducerOrientationSequence'),
array(0x0008,0x2246,'SQ','1','TransducerOrientationModifierSequence'),
array(0x0008,0x4000,'LT','1-n','IdentifyingComments'),
array(0x0010,0x0000,'UL','1','PatientGroupLength'),
array(0x0010,0x0010,'PN','1','PatientName'),
array(0x0010,0x0020,'LO','1','PatientID'),
array(0x0010,0x0021,'LO','1','IssuerOfPatientID'),
array(0x0010,0x0030,'DA','1','PatientBirthDate'),
array(0x0010,0x0032,'TM','1','PatientBirthTime'),
array(0x0010,0x0040,'CS','1','PatientSex'),
array(0x0010,0x0050,'SQ','1','PatientInsurancePlanCodeSequence'),
array(0x0010,0x1000,'LO','1-n','OtherPatientID'),
array(0x0010,0x1001,'PN','1-n','OtherPatientName'),
array(0x0010,0x1005,'PN','1','PatientBirthName'),
array(0x0010,0x1010,'AS','1','PatientAge'),
array(0x0010,0x1020,'DS','1','PatientSize'),
array(0x0010,0x1030,'DS','1','PatientWeight'),
array(0x0010,0x1040,'LO','1','PatientAddress'),
array(0x0010,0x1050,'LT','1-n','InsurancePlanIdentification'),
array(0x0010,0x1060,'PN','1','PatientMotherBirthName'),
array(0x0010,0x1080,'LO','1','MilitaryRank'),
array(0x0010,0x1081,'LO','1','BranchOfService'),
array(0x0010,0x1090,'LO','1','MedicalRecordLocator'),
array(0x0010,0x2000,'LO','1-n','MedicalAlerts'),
array(0x0010,0x2110,'LO','1-n','ContrastAllergies'),
array(0x0010,0x2150,'LO','1','CountryOfResidence'),
array(0x0010,0x2152,'LO','1','RegionOfResidence'),
array(0x0010,0x2154,'SH','1-n','PatientTelephoneNumber'),
array(0x0010,0x2160,'SH','1','EthnicGroup'),
array(0x0010,0x2180,'SH','1','Occupation'),
array(0x0010,0x21A0,'CS','1','SmokingStatus'),
array(0x0010,0x21B0,'LT','1','AdditionalPatientHistory'),
array(0x0010,0x21C0,'US','1','PregnancyStatus'),
array(0x0010,0x21D0,'DA','1','LastMenstrualDate'),
array(0x0010,0x21F0,'LO','1','PatientReligiousPreference'),
array(0x0010,0x4000,'LT','1','PatientComments'),
array(0x0018,0x0000,'UL','1','AcquisitionGroupLength'),
array(0x0018,0x0010,'LO','1','ContrastBolusAgent'),
array(0x0018,0x0012,'SQ','1','ContrastBolusAgentSequence'),
array(0x0018,0x0014,'SQ','1','ContrastBolusAdministrationRouteSequence'),
array(0x0018,0x0015,'CS','1','BodyPartExamined'),
array(0x0018,0x0020,'CS','1-n','ScanningSequence'),
array(0x0018,0x0021,'CS','1-n','SequenceVariant'),
array(0x0018,0x0022,'CS','1-n','ScanOptions'),
array(0x0018,0x0023,'CS','1','MRAcquisitionType'),
array(0x0018,0x0024,'SH','1','SequenceName'),
array(0x0018,0x0025,'CS','1','AngioFlag'),
array(0x0018,0x0026,'SQ','1','InterventionDrugInformationSequence'),
array(0x0018,0x0027,'TM','1','InterventionDrugStopTime'),
array(0x0018,0x0028,'DS','1','InterventionDrugDose'),
array(0x0018,0x0029,'SQ','1','InterventionalDrugSequence'),
array(0x0018,0x002A,'SQ','1','AdditionalDrugSequence'),
array(0x0018,0x0030,'LO','1-n','Radionuclide'),
array(0x0018,0x0031,'LO','1-n','Radiopharmaceutical'),
array(0x0018,0x0032,'DS','1','EnergyWindowCenterline'),
array(0x0018,0x0033,'DS','1-n','EnergyWindowTotalWidth'),
array(0x0018,0x0034,'LO','1','InterventionalDrugName'),
array(0x0018,0x0035,'TM','1','InterventionalDrugStartTime'),
array(0x0018,0x0036,'SQ','1','InterventionalTherapySequence'),
array(0x0018,0x0037,'CS','1','TherapyType'),
array(0x0018,0x0038,'CS','1','InterventionalStatus'),
array(0x0018,0x0039,'CS','1','TherapyDescription'),
array(0x0018,0x0040,'IS','1','CineRate'),
array(0x0018,0x0050,'DS','1','SliceThickness'),
array(0x0018,0x0060,'DS','1','KVP'),
array(0x0018,0x0070,'IS','1','CountsAccumulated'),
array(0x0018,0x0071,'CS','1','AcquisitionTerminationCondition'),
array(0x0018,0x0072,'DS','1','EffectiveSeriesDuration'),
array(0x0018,0x0073,'CS','1','AcquisitionStartCondition'),
array(0x0018,0x0074,'IS','1','AcquisitionStartConditionData'),
array(0x0018,0x0075,'IS','1','AcquisitionTerminationConditionData'),
array(0x0018,0x0080,'DS','1','RepetitionTime'),
array(0x0018,0x0081,'DS','1','EchoTime'),
array(0x0018,0x0082,'DS','1','InversionTime'),
array(0x0018,0x0083,'DS','1','NumberOfAverages'),
array(0x0018,0x0084,'DS','1','ImagingFrequency'),
array(0x0018,0x0085,'SH','1','ImagedNucleus'),
array(0x0018,0x0086,'IS','1-n','EchoNumber'),
array(0x0018,0x0087,'DS','1','MagneticFieldStrength'),
array(0x0018,0x0088,'DS','1','SpacingBetweenSlices'),
array(0x0018,0x0089,'IS','1','NumberOfPhaseEncodingSteps'),
array(0x0018,0x0090,'DS','1','DataCollectionDiameter'),
array(0x0018,0x0091,'IS','1','EchoTrainLength'),
array(0x0018,0x0093,'DS','1','PercentSampling'),
array(0x0018,0x0094,'DS','1','PercentPhaseFieldOfView'),
array(0x0018,0x0095,'DS','1','PixelBandwidth'),
array(0x0018,0x1000,'LO','1','DeviceSerialNumber'),
array(0x0018,0x1004,'LO','1','PlateID'),
array(0x0018,0x1010,'LO','1','SecondaryCaptureDeviceID'),
array(0x0018,0x1011,'LO','1','HardcopyCreationDeviceID'),
array(0x0018,0x1012,'DA','1','DateOfSecondaryCapture'),
array(0x0018,0x1014,'TM','1','TimeOfSecondaryCapture'),
array(0x0018,0x1016,'LO','1','SecondaryCaptureDeviceManufacturer'),
array(0x0018,0x1017,'LO','1','HardcopyDeviceManufacturer'),
array(0x0018,0x1018,'LO','1','SecondaryCaptureDeviceManufacturerModelName'),
array(0x0018,0x1019,'LO','1-n','SecondaryCaptureDeviceSoftwareVersion'),
array(0x0018,0x101A,'LO','1-n','HardcopyDeviceSoftwareVersion'),
array(0x0018,0x101B,'LO','1','HardcopyDeviceManfuacturersModelName'),
array(0x0018,0x1020,'LO','1-n','SoftwareVersion'),
array(0x0018,0x1022,'SH','1','VideoImageFormatAcquired'),
array(0x0018,0x1023,'LO','1','DigitalImageFormatAcquired'),
array(0x0018,0x1030,'LO','1','ProtocolName'),
array(0x0018,0x1040,'LO','1','ContrastBolusRoute'),
array(0x0018,0x1041,'DS','1','ContrastBolusVolume'),
array(0x0018,0x1042,'TM','1','ContrastBolusStartTime'),
array(0x0018,0x1043,'TM','1','ContrastBolusStopTime'),
array(0x0018,0x1044,'DS','1','ContrastBolusTotalDose'),
array(0x0018,0x1045,'IS','1-n','SyringeCounts'),
array(0x0018,0x1046,'DS','1-n','ContrastFlowRate'),
array(0x0018,0x1047,'DS','1-n','ContrastFlowDuration'),
array(0x0018,0x1048,'CS','1','ContrastBolusIngredient'),
array(0x0018,0x1049,'DS','1','ContrastBolusIngredientConcentration'),
array(0x0018,0x1050,'DS','1','SpatialResolution'),
array(0x0018,0x1060,'DS','1','TriggerTime'),
array(0x0018,0x1061,'LO','1','TriggerSourceOrType'),
array(0x0018,0x1062,'IS','1','NominalInterval'),
array(0x0018,0x1063,'DS','1','FrameTime'),
array(0x0018,0x1064,'LO','1','FramingType'),
array(0x0018,0x1065,'DS','1-n','FrameTimeVector'),
array(0x0018,0x1066,'DS','1','FrameDelay'),
array(0x0018,0x1067,'DS','1','ImageTriggerDelay'),
array(0x0018,0x1068,'DS','1','MultiplexGroupTimeOffset'),
array(0x0018,0x1069,'DS','1','TriggerTimeOffset'),
array(0x0018,0x106A,'CS','1','SynchronizationTrigger'),
array(0x0018,0x106C,'US','2','SynchronizationChannel'),
array(0x0018,0x106E,'UL','1','TriggerSamplePosition'),
array(0x0018,0x1070,'LO','1-n','RadionuclideRoute'),
array(0x0018,0x1071,'DS','1-n','RadionuclideVolume'),
array(0x0018,0x1072,'TM','1-n','RadionuclideStartTime'),
array(0x0018,0x1073,'TM','1-n','RadionuclideStopTime'),
array(0x0018,0x1074,'DS','1-n','RadionuclideTotalDose'),
array(0x0018,0x1075,'DS','1','RadionuclideHalfLife'),
array(0x0018,0x1076,'DS','1','RadionuclidePositronFraction'),
array(0x0018,0x1077,'DS','1','RadiopharmaceuticalSpecificActivity'),
array(0x0018,0x1080,'CS','1','BeatRejectionFlag'),
array(0x0018,0x1081,'IS','1','LowRRValue'),
array(0x0018,0x1082,'IS','1','HighRRValue'),
array(0x0018,0x1083,'IS','1','IntervalsAcquired'),
array(0x0018,0x1084,'IS','1','IntervalsRejected'),
array(0x0018,0x1085,'LO','1','PVCRejection'),
array(0x0018,0x1086,'IS','1','SkipBeats'),
array(0x0018,0x1088,'IS','1','HeartRate'),
array(0x0018,0x1090,'IS','1','CardiacNumberOfImages'),
array(0x0018,0x1094,'IS','1','TriggerWindow'),
array(0x0018,0x1100,'DS','1','ReconstructionDiameter'),
array(0x0018,0x1110,'DS','1','DistanceSourceToDetector'),
array(0x0018,0x1111,'DS','1','DistanceSourceToPatient'),
array(0x0018,0x1114,'DS','1','EstimatedRadiographicMagnificationFactor'),
array(0x0018,0x1120,'DS','1','GantryDetectorTilt'),
array(0x0018,0x1121,'DS','1','GantryDetectorSlew'),
array(0x0018,0x1130,'DS','1','TableHeight'),
array(0x0018,0x1131,'DS','1','TableTraverse'),
array(0x0018,0x1134,'DS','1','TableMotion'),
array(0x0018,0x1135,'DS','1-n','TableVerticalIncrement'),
array(0x0018,0x1136,'DS','1-n','TableLateralIncrement'),
array(0x0018,0x1137,'DS','1-n','TableLongitudinalIncrement'),
array(0x0018,0x1138,'DS','1','TableAngle'),
array(0x0018,0x113A,'CS','1','TableType'),
array(0x0018,0x1140,'CS','1','RotationDirection'),
array(0x0018,0x1141,'DS','1','AngularPosition'),
array(0x0018,0x1142,'DS','1-n','RadialPosition'),
array(0x0018,0x1143,'DS','1','ScanArc'),
array(0x0018,0x1144,'DS','1','AngularStep'),
array(0x0018,0x1145,'DS','1','CenterOfRotationOffset'),
array(0x0018,0x1146,'DS','1-n','RotationOffset'),
array(0x0018,0x1147,'CS','1','FieldOfViewShape'),
array(0x0018,0x1149,'IS','2','FieldOfViewDimension'),
array(0x0018,0x1150,'IS','1','ExposureTime'),
array(0x0018,0x1151,'IS','1','XrayTubeCurrent'),
array(0x0018,0x1152,'IS','1','Exposure'),
array(0x0018,0x1153,'IS','1','ExposureinuAs'),
array(0x0018,0x1154,'DS','1','AveragePulseWidth'),
array(0x0018,0x1155,'CS','1','RadiationSetting'),
array(0x0018,0x1156,'CS','1','RectificationType'),
array(0x0018,0x115A,'CS','1','RadiationMode'),
array(0x0018,0x115E,'DS','1','ImageAreaDoseProduct'),
array(0x0018,0x1160,'SH','1','FilterType'),
array(0x0018,0x1161,'LO','1-n','TypeOfFilters'),
array(0x0018,0x1162,'DS','1','IntensifierSize'),
array(0x0018,0x1164,'DS','2','ImagerPixelSpacing'),
array(0x0018,0x1166,'CS','1','Grid'),
array(0x0018,0x1170,'IS','1','GeneratorPower'),
array(0x0018,0x1180,'SH','1','CollimatorGridName'),
array(0x0018,0x1181,'CS','1','CollimatorType'),
array(0x0018,0x1182,'IS','1','FocalDistance'),
array(0x0018,0x1183,'DS','1','XFocusCenter'),
array(0x0018,0x1184,'DS','1','YFocusCenter'),
array(0x0018,0x1190,'DS','1-n','FocalSpot'),
array(0x0018,0x1191,'CS','1','AnodeTargetMaterial'),
array(0x0018,0x11A0,'DS','1','BodyPartThickness'),
array(0x0018,0x11A2,'DS','1','CompressionForce'),
array(0x0018,0x1200,'DA','1-n','DateOfLastCalibration'),
array(0x0018,0x1201,'TM','1-n','TimeOfLastCalibration'),
array(0x0018,0x1210,'SH','1-n','ConvolutionKernel'),
array(0x0018,0x1240,'IS','1-n','UpperLowerPixelValues'),
array(0x0018,0x1242,'IS','1','ActualFrameDuration'),
array(0x0018,0x1243,'IS','1','CountRate'),
array(0x0018,0x1244,'US','1','PreferredPlaybackSequencing'),
array(0x0018,0x1250,'SH','1','ReceivingCoil'),
array(0x0018,0x1251,'SH','1','TransmittingCoil'),
array(0x0018,0x1260,'SH','1','PlateType'),
array(0x0018,0x1261,'LO','1','PhosphorType'),
array(0x0018,0x1300,'IS','1','ScanVelocity'),
array(0x0018,0x1301,'CS','1-n','WholeBodyTechnique'),
array(0x0018,0x1302,'IS','1','ScanLength'),
array(0x0018,0x1310,'US','4','AcquisitionMatrix'),
array(0x0018,0x1312,'CS','1','PhaseEncodingDirection'),
array(0x0018,0x1314,'DS','1','FlipAngle'),
array(0x0018,0x1315,'CS','1','VariableFlipAngleFlag'),
array(0x0018,0x1316,'DS','1','SAR'),
array(0x0018,0x1318,'DS','1','dBdt'),
array(0x0018,0x1400,'LO','1','AcquisitionDeviceProcessingDescription'),
array(0x0018,0x1401,'LO','1','AcquisitionDeviceProcessingCode'),
array(0x0018,0x1402,'CS','1','CassetteOrientation'),
array(0x0018,0x1403,'CS','1','CassetteSize'),
array(0x0018,0x1404,'US','1','ExposuresOnPlate'),
array(0x0018,0x1405,'IS','1','RelativeXrayExposure'),
array(0x0018,0x1450,'DS','1','ColumnAngulation'),
array(0x0018,0x1460,'DS','1','TomoLayerHeight'),
array(0x0018,0x1470,'DS','1','TomoAngle'),
array(0x0018,0x1480,'DS','1','TomoTime'),
array(0x0018,0x1490,'CS','1','TomoType'),
array(0x0018,0x1491,'CS','1','TomoClass'),
array(0x0018,0x1495,'IS','1','NumberofTomosynthesisSourceImages'),
array(0x0018,0x1500,'CS','1','PositionerMotion'),
array(0x0018,0x1508,'CS','1','PositionerType'),
array(0x0018,0x1510,'DS','1','PositionerPrimaryAngle'),
array(0x0018,0x1511,'DS','1','PositionerSecondaryAngle'),
array(0x0018,0x1520,'DS','1-n','PositionerPrimaryAngleIncrement'),
array(0x0018,0x1521,'DS','1-n','PositionerSecondaryAngleIncrement'),
array(0x0018,0x1530,'DS','1','DetectorPrimaryAngle'),
array(0x0018,0x1531,'DS','1','DetectorSecondaryAngle'),
array(0x0018,0x1600,'CS','3','ShutterShape'),
array(0x0018,0x1602,'IS','1','ShutterLeftVerticalEdge'),
array(0x0018,0x1604,'IS','1','ShutterRightVerticalEdge'),
array(0x0018,0x1606,'IS','1','ShutterUpperHorizontalEdge'),
array(0x0018,0x1608,'IS','1','ShutterLowerHorizontalEdge'),
array(0x0018,0x1610,'IS','1','CenterOfCircularShutter'),
array(0x0018,0x1612,'IS','1','RadiusOfCircularShutter'),
array(0x0018,0x1620,'IS','1-n','VerticesOfPolygonalShutter'),
array(0x0018,0x1622,'US','1','ShutterPresentationValue'),
array(0x0018,0x1623,'US','1','ShutterOverlayGroup'),
array(0x0018,0x1700,'CS','3','CollimatorShape'),
array(0x0018,0x1702,'IS','1','CollimatorLeftVerticalEdge'),
array(0x0018,0x1704,'IS','1','CollimatorRightVerticalEdge'),
array(0x0018,0x1706,'IS','1','CollimatorUpperHorizontalEdge'),
array(0x0018,0x1708,'IS','1','CollimatorLowerHorizontalEdge'),
array(0x0018,0x1710,'IS','1','CenterOfCircularCollimator'),
array(0x0018,0x1712,'IS','1','RadiusOfCircularCollimator'),
array(0x0018,0x1720,'IS','1-n','VerticesOfPolygonalCollimator'),
array(0x0018,0x1800,'CS','1','AcquisitionTimeSynchronized'),
array(0x0018,0x1801,'SH','1','TimeSource'),
array(0x0018,0x1802,'CS','1','TimeDistributionProtocol'),
array(0x0018,0x1810,'DT','1','AcquisitionTimestamp'),
array(0x0018,0x4000,'LT','1-n','AcquisitionComments'),
array(0x0018,0x5000,'SH','1-n','OutputPower'),
array(0x0018,0x5010,'LO','3','TransducerData'),
array(0x0018,0x5012,'DS','1','FocusDepth'),
array(0x0018,0x5020,'LO','1','PreprocessingFunction'),
array(0x0018,0x5021,'LO','1','PostprocessingFunction'),
array(0x0018,0x5022,'DS','1','MechanicalIndex'),
array(0x0018,0x5024,'DS','1','ThermalIndex'),
array(0x0018,0x5026,'DS','1','CranialThermalIndex'),
array(0x0018,0x5027,'DS','1','SoftTissueThermalIndex'),
array(0x0018,0x5028,'DS','1','SoftTissueFocusThermalIndex'),
array(0x0018,0x5029,'DS','1','SoftTissueSurfaceThermalIndex'),
array(0x0018,0x5030,'DS','1','DynamicRange'),
array(0x0018,0x5040,'DS','1','TotalGain'),
array(0x0018,0x5050,'IS','1','DepthOfScanField'),
array(0x0018,0x5100,'CS','1','PatientPosition'),
array(0x0018,0x5101,'CS','1','ViewPosition'),
array(0x0018,0x5104,'SQ','1','ProjectionEponymousNameCodeSequence'),
array(0x0018,0x5210,'DS','6','ImageTransformationMatrix'),
array(0x0018,0x5212,'DS','3','ImageTranslationVector'),
array(0x0018,0x6000,'DS','1','Sensitivity'),
array(0x0018,0x6011,'SQ','1','SequenceOfUltrasoundRegions'),
array(0x0018,0x6012,'US','1','RegionSpatialFormat'),
array(0x0018,0x6014,'US','1','RegionDataType'),
array(0x0018,0x6016,'UL','1','RegionFlags'),
array(0x0018,0x6018,'UL','1','RegionLocationMinX0'),
array(0x0018,0x601A,'UL','1','RegionLocationMinY0'),
array(0x0018,0x601C,'UL','1','RegionLocationMaxX1'),
array(0x0018,0x601E,'UL','1','RegionLocationMaxY1'),
array(0x0018,0x6020,'SL','1','ReferencePixelX0'),
array(0x0018,0x6022,'SL','1','ReferencePixelY0'),
array(0x0018,0x6024,'US','1','PhysicalUnitsXDirection'),
array(0x0018,0x6026,'US','1','PhysicalUnitsYDirection'),
array(0x0018,0x6028,'FD','1','ReferencePixelPhysicalValueX'),
array(0x0018,0x602A,'FD','1','ReferencePixelPhysicalValueY'),
array(0x0018,0x602C,'FD','1','PhysicalDeltaX'),
array(0x0018,0x602E,'FD','1','PhysicalDeltaY'),
array(0x0018,0x6030,'UL','1','TransducerFrequency'),
array(0x0018,0x6031,'CS','1','TransducerType'),
array(0x0018,0x6032,'UL','1','PulseRepetitionFrequency'),
array(0x0018,0x6034,'FD','1','DopplerCorrectionAngle'),
array(0x0018,0x6036,'FD','1','SteeringAngle'),
array(0x0018,0x6038,'UL','1','DopplerSampleVolumeXPosition'),
array(0x0018,0x603A,'UL','1','DopplerSampleVolumeYPosition'),
array(0x0018,0x603C,'UL','1','TMLinePositionX0'),
array(0x0018,0x603E,'UL','1','TMLinePositionY0'),
array(0x0018,0x6040,'UL','1','TMLinePositionX1'),
array(0x0018,0x6042,'UL','1','TMLinePositionY1'),
array(0x0018,0x6044,'US','1','PixelComponentOrganization'),
array(0x0018,0x6046,'UL','1','PixelComponentMask'),
array(0x0018,0x6048,'UL','1','PixelComponentRangeStart'),
array(0x0018,0x604A,'UL','1','PixelComponentRangeStop'),
array(0x0018,0x604C,'US','1','PixelComponentPhysicalUnits'),
array(0x0018,0x604E,'US','1','PixelComponentDataType'),
array(0x0018,0x6050,'UL','1','NumberOfTableBreakPoints'),
array(0x0018,0x6052,'UL','1-n','TableOfXBreakPoints'),
array(0x0018,0x6054,'FD','1-n','TableOfYBreakPoints'),
array(0x0018,0x6056,'UL','1','NumberOfTableEntries'),
array(0x0018,0x6058,'UL','1-n','TableOfPixelValues'),
array(0x0018,0x605A,'FL','1-n','TableOfParameterValues'),
array(0x0018,0x7000,'CS','1','DetectorConditionsNominalFlag'),
array(0x0018,0x7001,'DS','1','DetectorTemperature'),
array(0x0018,0x7004,'CS','1','DetectorType'),
array(0x0018,0x7005,'CS','1','DetectorConfiguration'),
array(0x0018,0x7006,'LT','1','DetectorDescription'),
array(0x0018,0x7008,'LT','1','DetectorMode'),
array(0x0018,0x700A,'SH','1','DetectorID'),
array(0x0018,0x700C,'DA','1','DateofLastDetectorCalibration'),
array(0x0018,0x700E,'TM','1','TimeofLastDetectorCalibration'),
array(0x0018,0x7010,'IS','1','ExposuresOnDetectorSinceLastCalibration'),
array(0x0018,0x7011,'IS','1','ExposuresOnDetectorSinceManufactured'),
array(0x0018,0x7012,'DS','1','DetectorTimeSinceLastExposure'),
array(0x0018,0x7014,'DS','1','DetectorActiveTime'),
array(0x0018,0x7016,'DS','1','DetectorActivationOffsetFromExposure'),
array(0x0018,0x701A,'DS','2','DetectorBinning'),
array(0x0018,0x7020,'DS','2','DetectorElementPhysicalSize'),
array(0x0018,0x7022,'DS','2','DetectorElementSpacing'),
array(0x0018,0x7024,'CS','1','DetectorActiveShape'),
array(0x0018,0x7026,'DS','1-2','DetectorActiveDimensions'),
array(0x0018,0x7028,'DS','2','DetectorActiveOrigin'),
array(0x0018,0x7030,'DS','2','FieldofViewOrigin'),
array(0x0018,0x7032,'DS','1','FieldofViewRotation'),
array(0x0018,0x7034,'CS','1','FieldofViewHorizontalFlip'),
array(0x0018,0x7040,'LT','1','GridAbsorbingMaterial'),
array(0x0018,0x7041,'LT','1','GridSpacingMaterial'),
array(0x0018,0x7042,'DS','1','GridThickness'),
array(0x0018,0x7044,'DS','1','GridPitch'),
array(0x0018,0x7046,'IS','2','GridAspectRatio'),
array(0x0018,0x7048,'DS','1','GridPeriod'),
array(0x0018,0x704C,'DS','1','GridFocalDistance'),
array(0x0018,0x7050,'LT','1-n','FilterMaterial'),
array(0x0018,0x7052,'DS','1-n','FilterThicknessMinimum'),
array(0x0018,0x7054,'DS','1-n','FilterThicknessMaximum'),
array(0x0018,0x7060,'CS','1','ExposureControlMode'),
array(0x0018,0x7062,'LT','1','ExposureControlModeDescription'),
array(0x0018,0x7064,'CS','1','ExposureStatus'),
array(0x0018,0x7065,'DS','1','PhototimerSetting'),
array(0x0020,0x0000,'UL','1','ImageGroupLength'),
array(0x0020,0x000D,'UI','1','StudyInstanceUID'),
array(0x0020,0x000E,'UI','1','SeriesInstanceUID'),
array(0x0020,0x0010,'SH','1','StudyID'),
array(0x0020,0x0011,'IS','1','SeriesNumber'),
array(0x0020,0x0012,'IS','1','AcquisitionNumber'),
array(0x0020,0x0013,'IS','1','ImageNumber'),
array(0x0020,0x0014,'IS','1','IsotopeNumber'),
array(0x0020,0x0015,'IS','1','PhaseNumber'),
array(0x0020,0x0016,'IS','1','IntervalNumber'),
array(0x0020,0x0017,'IS','1','TimeSlotNumber'),
array(0x0020,0x0018,'IS','1','AngleNumber'),
array(0x0020,0x0019,'IS','1','ItemNumber'),
array(0x0020,0x0020,'CS','2','PatientOrientation'),
array(0x0020,0x0022,'IS','1','OverlayNumber'),
array(0x0020,0x0024,'IS','1','CurveNumber'),
array(0x0020,0x0026,'IS','1','LUTNumber'),
array(0x0020,0x0030,'DS','3','ImagePosition'),
array(0x0020,0x0032,'DS','3','ImagePositionPatient'),
array(0x0020,0x0035,'DS','6','ImageOrientation'),
array(0x0020,0x0037,'DS','6','ImageOrientationPatient'),
array(0x0020,0x0050,'DS','1','Location'),
array(0x0020,0x0052,'UI','1','FrameOfReferenceUID'),
array(0x0020,0x0060,'CS','1','Laterality'),
array(0x0020,0x0062,'CS','1','ImageLaterality'),
array(0x0020,0x0070,'LT','1','ImageGeometryType'),
array(0x0020,0x0080,'CS','1-n','MaskingImage'),
array(0x0020,0x0100,'IS','1','TemporalPositionIdentifier'),
array(0x0020,0x0105,'IS','1','NumberOfTemporalPositions'),
array(0x0020,0x0110,'DS','1','TemporalResolution'),
array(0x0020,0x0200,'UI','1','SynchronizationFrameofReferenceUID'),
array(0x0020,0x1000,'IS','1','SeriesInStudy'),
array(0x0020,0x1001,'IS','1','AcquisitionsInSeries'),
array(0x0020,0x1002,'IS','1','ImagesInAcquisition'),
array(0x0020,0x1003,'IS','1','ImagesInSeries'),
array(0x0020,0x1004,'IS','1','AcquisitionsInStudy'),
array(0x0020,0x1005,'IS','1','ImagesInStudy'),
array(0x0020,0x1020,'CS','1-n','Reference'),
array(0x0020,0x1040,'LO','1','PositionReferenceIndicator'),
array(0x0020,0x1041,'DS','1','SliceLocation'),
array(0x0020,0x1070,'IS','1-n','OtherStudyNumbers'),
array(0x0020,0x1200,'IS','1','NumberOfPatientRelatedStudies'),
array(0x0020,0x1202,'IS','1','NumberOfPatientRelatedSeries'),
array(0x0020,0x1204,'IS','1','NumberOfPatientRelatedImages'),
array(0x0020,0x1206,'IS','1','NumberOfStudyRelatedSeries'),
array(0x0020,0x1208,'IS','1','NumberOfStudyRelatedImages'),
array(0x0020,0x1209,'IS','1','NumberOfSeriesRelatedInstances'),
array(0x0020,0x3100,'CS','1-n','SourceImageID'),
array(0x0020,0x3401,'CS','1','ModifyingDeviceID'),
array(0x0020,0x3402,'CS','1','ModifiedImageID'),
array(0x0020,0x3403,'DA','1','ModifiedImageDate'),
array(0x0020,0x3404,'LO','1','ModifyingDeviceManufacturer'),
array(0x0020,0x3405,'TM','1','ModifiedImageTime'),
array(0x0020,0x3406,'LT','1','ModifiedImageDescription'),
array(0x0020,0x4000,'LT','1','ImageComments'),
array(0x0020,0x5000,'AT','1-n','OriginalImageIdentification'),
array(0x0020,0x5002,'CS','1-n','OriginalImageIdentificationNomenclature'),
array(0x0028,0x0000,'UL','1','ImagePresentationGroupLength'),
array(0x0028,0x0002,'US','1','SamplesPerPixel'),
array(0x0028,0x0004,'CS','1','PhotometricInterpretation'),
array(0x0028,0x0005,'US','1','ImageDimensions'),
array(0x0028,0x0006,'US','1','PlanarConfiguration'),
array(0x0028,0x0008,'IS','1','NumberOfFrames'),
array(0x0028,0x0009,'AT','1','FrameIncrementPointer'),
array(0x0028,0x0010,'US','1','Rows'),
array(0x0028,0x0011,'US','1','Columns'),
array(0x0028,0x0012,'US','1','Planes'),
array(0x0028,0x0014,'US','1','UltrasoundColorDataPresent'),
array(0x0028,0x0030,'DS','2','PixelSpacing'),
array(0x0028,0x0031,'DS','2','ZoomFactor'),
array(0x0028,0x0032,'DS','2','ZoomCenter'),
array(0x0028,0x0034,'IS','2','PixelAspectRatio'),
array(0x0028,0x0040,'CS','1','ImageFormat'),
array(0x0028,0x0050,'LT','1-n','ManipulatedImage'),
array(0x0028,0x0051,'CS','1','CorrectedImage'),
array(0x0028,0x005F,'CS','1','CompressionRecognitionCode'),
array(0x0028,0x0060,'CS','1','CompressionCode'),
array(0x0028,0x0061,'SH','1','CompressionOriginator'),
array(0x0028,0x0062,'SH','1','CompressionLabel'),
array(0x0028,0x0063,'SH','1','CompressionDescription'),
array(0x0028,0x0065,'CS','1-n','CompressionSequence'),
array(0x0028,0x0066,'AT','1-n','CompressionStepPointers'),
array(0x0028,0x0068,'US','1','RepeatInterval'),
array(0x0028,0x0069,'US','1','BitsGrouped'),
array(0x0028,0x0070,'US','1-n','PerimeterTable'),
array(0x0028,0x0071,'XS','1','PerimeterValue'),
array(0x0028,0x0080,'US','1','PredictorRows'),
array(0x0028,0x0081,'US','1','PredictorColumns'),
array(0x0028,0x0082,'US','1-n','PredictorConstants'),
array(0x0028,0x0090,'CS','1','BlockedPixels'),
array(0x0028,0x0091,'US','1','BlockRows'),
array(0x0028,0x0092,'US','1','BlockColumns'),
array(0x0028,0x0093,'US','1','RowOverlap'),
array(0x0028,0x0094,'US','1','ColumnOverlap'),
array(0x0028,0x0100,'US','1','BitsAllocated'),
array(0x0028,0x0101,'US','1','BitsStored'),
array(0x0028,0x0102,'US','1','HighBit'),
array(0x0028,0x0103,'US','1','PixelRepresentation'),
array(0x0028,0x0104,'XS','1','SmallestValidPixelValue'),
array(0x0028,0x0105,'XS','1','LargestValidPixelValue'),
array(0x0028,0x0106,'XS','1','SmallestImagePixelValue'),
array(0x0028,0x0107,'XS','1','LargestImagePixelValue'),
array(0x0028,0x0108,'XS','1','SmallestPixelValueInSeries'),
array(0x0028,0x0109,'XS','1','LargestPixelValueInSeries'),
array(0x0028,0x0110,'XS','1','SmallestPixelValueInPlane'),
array(0x0028,0x0111,'XS','1','LargestPixelValueInPlane'),
array(0x0028,0x0120,'XS','1','PixelPaddingValue'),
array(0x0028,0x0200,'US','1','ImageLocation'),
array(0x0028,0x0300,'CS','1','QualityControlImage'),
array(0x0028,0x0301,'CS','1','BurnedInAnnotation'),
array(0x0028,0x0400,'CS','1','TransformLabel'),
array(0x0028,0x0401,'CS','1','TransformVersionNumber'),
array(0x0028,0x0402,'US','1','NumberOfTransformSteps'),
array(0x0028,0x0403,'CS','1-n','SequenceOfCompressedData'),
array(0x0028,0x0404,'AT','1-n','DetailsOfCoefficients'),
array(0x0028,0x0410,'US','1','RowsForNthOrderCoefficients'),
array(0x0028,0x0411,'US','1','ColumnsForNthOrderCoefficients'),
array(0x0028,0x0412,'CS','1-n','CoefficientCoding'),
array(0x0028,0x0413,'AT','1-n','CoefficientCodingPointers'),
array(0x0028,0x0700,'CS','1','DCTLabel'),
array(0x0028,0x0701,'CS','1-n','DataBlockDescription'),
array(0x0028,0x0702,'AT','1-n','DataBlock'),
array(0x0028,0x0710,'US','1','NormalizationFactorFormat'),
array(0x0028,0x0720,'US','1','ZonalMapNumberFormat'),
array(0x0028,0x0721,'AT','1-n','ZonalMapLocation'),
array(0x0028,0x0722,'US','1','ZonalMapFormat'),
array(0x0028,0x0730,'US','1','AdaptiveMapFormat'),
array(0x0028,0x0740,'US','1','CodeNumberFormat'),
array(0x0028,0x0800,'CS','1-n','CodeLabel'),
array(0x0028,0x0802,'US','1','NumberOfTables'),
array(0x0028,0x0803,'AT','1-n','CodeTableLocation'),
array(0x0028,0x0804,'US','1','BitsForCodeWord'),
array(0x0028,0x0808,'AT','1-n','ImageDataLocation'),
array(0x0028,0x1040,'CS','1','PixelIntensityRelationship'),
array(0x0028,0x1041,'SS','1','PixelIntensityRelationshipSign'),
array(0x0028,0x1050,'DS','1-n','WindowCenter'),
array(0x0028,0x1051,'DS','1-n','WindowWidth'),
array(0x0028,0x1052,'DS','1','RescaleIntercept'),
array(0x0028,0x1053,'DS','1','RescaleSlope'),
array(0x0028,0x1054,'LO','1','RescaleType'),
array(0x0028,0x1055,'LO','1-n','WindowCenterWidthExplanation'),
array(0x0028,0x1080,'CS','1','GrayScale'),
array(0x0028,0x1090,'CS','1','RecommendedViewingMode'),
array(0x0028,0x1100,'XS','3','GrayLookupTableDescriptor'),
array(0x0028,0x1101,'XS','3','RedPaletteColorLookupTableDescriptor'),
array(0x0028,0x1102,'XS','3','GreenPaletteColorLookupTableDescriptor'),
array(0x0028,0x1103,'XS','3','BluePaletteColorLookupTableDescriptor'),
array(0x0028,0x1111,'US','4','LargeRedPaletteColorLookupTableDescriptor'),
array(0x0028,0x1112,'US','4','LargeGreenPaletteColorLookupTabe'),
array(0x0028,0x1113,'US','4','LargeBluePaletteColorLookupTabl'),
array(0x0028,0x1199,'UI','1','PaletteColorLookupTableUID'),
array(0x0028,0x1200,'XS','1-n','GrayLookupTableData'),
array(0x0028,0x1201,'XS','1-n','RedPaletteColorLookupTableData'),
array(0x0028,0x1202,'XS','1-n','GreenPaletteColorLookupTableData'),
array(0x0028,0x1203,'XS','1-n','BluePaletteColorLookupTableData'),
array(0x0028,0x1211,'OW','1','LargeRedPaletteColorLookupTableData'),
array(0x0028,0x1212,'OW','1','LargeGreenPaletteColorLookupTableData'),
array(0x0028,0x1213,'OW','1','LargeBluePaletteColorLookupTableData'),
array(0x0028,0x1214,'UI','1','LargePaletteColorLookupTableUID'),
array(0x0028,0x1221,'OW','1','SegmentedRedPaletteColorLookupTableData'),
array(0x0028,0x1222,'OW','1','SegmentedGreenPaletteColorLookupTableData'),
array(0x0028,0x1223,'OW','1','SegmentedBluePaletteColorLookupTableData'),
array(0x0028,0x1300,'CS','1','ImplantPresent'),
array(0x0028,0x2110,'CS','1','LossyImageCompression'),
array(0x0028,0x2112,'DS','1-n','LossyImageCompressionRatio'),
array(0x0028,0x3000,'SQ','1','ModalityLUTSequence'),
array(0x0028,0x3002,'XS','3','LUTDescriptor'),
array(0x0028,0x3003,'LO','1','LUTExplanation'),
array(0x0028,0x3004,'LO','1','ModalityLUTType'),
array(0x0028,0x3006,'XS','1-n','LUTData'),
array(0x0028,0x3010,'SQ','1','VOILUTSequence'),
array(0x0028,0x3110,'SQ','1','SoftcopyVOILUTSequence'),
array(0x0028,0x4000,'LT','1-n','ImagePresentationComments'),
array(0x0028,0x5000,'SQ','1','BiPlaneAcquisitionSequence'),
array(0x0028,0x6010,'US','1','RepresentativeFrameNumber'),
array(0x0028,0x6020,'US','1-n','FrameNumbersOfInterest'),
array(0x0028,0x6022,'LO','1-n','FrameOfInterestDescription'),
array(0x0028,0x6030,'US','1-n','MaskPointer'),
array(0x0028,0x6040,'US','1-n','RWavePointer'),
array(0x0028,0x6100,'SQ','1','MaskSubtractionSequence'),
array(0x0028,0x6101,'CS','1','MaskOperation'),
array(0x0028,0x6102,'US','1-n','ApplicableFrameRange'),
array(0x0028,0x6110,'US','1-n','MaskFrameNumbers'),
array(0x0028,0x6112,'US','1','ContrastFrameAveraging'),
array(0x0028,0x6114,'FL','2','MaskSubPixelShift'),
array(0x0028,0x6120,'SS','1','TIDOffset'),
array(0x0028,0x6190,'ST','1','MaskOperationExplanation'),
array(0x0032,0x0000,'UL','1','StudyGroupLength'),
array(0x0032,0x000A,'CS','1','StudyStatusID'),
array(0x0032,0x000C,'CS','1','StudyPriorityID'),
array(0x0032,0x0012,'LO','1','StudyIDIssuer'),
array(0x0032,0x0032,'DA','1','StudyVerifiedDate'),
array(0x0032,0x0033,'TM','1','StudyVerifiedTime'),
array(0x0032,0x0034,'DA','1','StudyReadDate'),
array(0x0032,0x0035,'TM','1','StudyReadTime'),
array(0x0032,0x1000,'DA','1','ScheduledStudyStartDate'),
array(0x0032,0x1001,'TM','1','ScheduledStudyStartTime'),
array(0x0032,0x1010,'DA','1','ScheduledStudyStopDate'),
array(0x0032,0x1011,'TM','1','ScheduledStudyStopTime'),
array(0x0032,0x1020,'LO','1','ScheduledStudyLocation'),
array(0x0032,0x1021,'AE','1-n','ScheduledStudyLocationAETitle'),
array(0x0032,0x1030,'LO','1','ReasonForStudy'),
array(0x0032,0x1032,'PN','1','RequestingPhysician'),
array(0x0032,0x1033,'LO','1','RequestingService'),
array(0x0032,0x1040,'DA','1','StudyArrivalDate'),
array(0x0032,0x1041,'TM','1','StudyArrivalTime'),
array(0x0032,0x1050,'DA','1','StudyCompletionDate'),
array(0x0032,0x1051,'TM','1','StudyCompletionTime'),
array(0x0032,0x1055,'CS','1','StudyComponentStatusID'),
array(0x0032,0x1060,'LO','1','RequestedProcedureDescription'),
array(0x0032,0x1064,'SQ','1','RequestedProcedureCodeSequence'),
array(0x0032,0x1070,'LO','1','RequestedContrastAgent'),
array(0x0032,0x4000,'LT','1','StudyComments'),
array(0x0038,0x0000,'UL','1','VisitGroupLength'),
array(0x0038,0x0004,'SQ','1','ReferencedPatientAliasSequence'),
array(0x0038,0x0008,'CS','1','VisitStatusID'),
array(0x0038,0x0010,'LO','1','AdmissionID'),
array(0x0038,0x0011,'LO','1','IssuerOfAdmissionID'),
array(0x0038,0x0016,'LO','1','RouteOfAdmissions'),
array(0x0038,0x001A,'DA','1','ScheduledAdmissionDate'),
array(0x0038,0x001B,'TM','1','ScheduledAdmissionTime'),
array(0x0038,0x001C,'DA','1','ScheduledDischargeDate'),
array(0x0038,0x001D,'TM','1','ScheduledDischargeTime'),
array(0x0038,0x001E,'LO','1','ScheduledPatientInstitutionResidence'),
array(0x0038,0x0020,'DA','1','AdmittingDate'),
array(0x0038,0x0021,'TM','1','AdmittingTime'),
array(0x0038,0x0030,'DA','1','DischargeDate'),
array(0x0038,0x0032,'TM','1','DischargeTime'),
array(0x0038,0x0040,'LO','1','DischargeDiagnosisDescription'),
array(0x0038,0x0044,'SQ','1','DischargeDiagnosisCodeSequence'),
array(0x0038,0x0050,'LO','1','SpecialNeeds'),
array(0x0038,0x0300,'LO','1','CurrentPatientLocation'),
array(0x0038,0x0400,'LO','1','PatientInstitutionResidence'),
array(0x0038,0x0500,'LO','1','PatientState'),
array(0x0038,0x4000,'LT','1','VisitComments'),
array(0x003A,0x0004,'CS','1','WaveformOriginality'),
array(0x003A,0x0005,'US','1','NumberofChannels'),
array(0x003A,0x0010,'UL','1','NumberofSamples'),
array(0x003A,0x001A,'DS','1','SamplingFrequency'),
array(0x003A,0x0020,'SH','1','MultiplexGroupLabel'),
array(0x003A,0x0200,'SQ','1','ChannelDefinitionSequence'),
array(0x003A,0x0202,'IS','1','WVChannelNumber'),
array(0x003A,0x0203,'SH','1','ChannelLabel'),
array(0x003A,0x0205,'CS','1-n','ChannelStatus'),
array(0x003A,0x0208,'SQ','1','ChannelSourceSequence'),
array(0x003A,0x0209,'SQ','1','ChannelSourceModifiersSequence'),
array(0x003A,0x020A,'SQ','1','SourceWaveformSequence'),
array(0x003A,0x020C,'LO','1','ChannelDerivationDescription'),
array(0x003A,0x0210,'DS','1','ChannelSensitivity'),
array(0x003A,0x0211,'SQ','1','ChannelSensitivityUnits'),
array(0x003A,0x0212,'DS','1','ChannelSensitivityCorrectionFactor'),
array(0x003A,0x0213,'DS','1','ChannelBaseline'),
array(0x003A,0x0214,'DS','1','ChannelTimeSkew'),
array(0x003A,0x0215,'DS','1','ChannelSampleSkew'),
array(0x003A,0x0218,'DS','1','ChannelOffset'),
array(0x003A,0x021A,'US','1','WaveformBitsStored'),
array(0x003A,0x0220,'DS','1','FilterLowFrequency'),
array(0x003A,0x0221,'DS','1','FilterHighFrequency'),
array(0x003A,0x0222,'DS','1','NotchFilterFrequency'),
array(0x003A,0x0223,'DS','1','NotchFilterBandwidth'),
array(0x0040,0x0000,'UL','1','ModalityWorklistGroupLength'),
array(0x0040,0x0001,'AE','1','ScheduledStationAETitle'),
array(0x0040,0x0002,'DA','1','ScheduledProcedureStepStartDate'),
array(0x0040,0x0003,'TM','1','ScheduledProcedureStepStartTime'),
array(0x0040,0x0004,'DA','1','ScheduledProcedureStepEndDate'),
array(0x0040,0x0005,'TM','1','ScheduledProcedureStepEndTime'),
array(0x0040,0x0006,'PN','1','ScheduledPerformingPhysicianName'),
array(0x0040,0x0007,'LO','1','ScheduledProcedureStepDescription'),
array(0x0040,0x0008,'SQ','1','ScheduledProcedureStepCodeSequence'),
array(0x0040,0x0009,'SH','1','ScheduledProcedureStepID'),
array(0x0040,0x0010,'SH','1','ScheduledStationName'),
array(0x0040,0x0011,'SH','1','ScheduledProcedureStepLocation'),
array(0x0040,0x0012,'LO','1','ScheduledPreOrderOfMedication'),
array(0x0040,0x0020,'CS','1','ScheduledProcedureStepStatus'),
array(0x0040,0x0100,'SQ','1-n','ScheduledProcedureStepSequence'),
array(0x0040,0x0220,'SQ','1','ReferencedStandaloneSOPInstanceSequence'),
array(0x0040,0x0241,'AE','1','PerformedStationAETitle'),
array(0x0040,0x0242,'SH','1','PerformedStationName'),
array(0x0040,0x0243,'SH','1','PerformedLocation'),
array(0x0040,0x0244,'DA','1','PerformedProcedureStepStartDate'),
array(0x0040,0x0245,'TM','1','PerformedProcedureStepStartTime'),
array(0x0040,0x0250,'DA','1','PerformedProcedureStepEndDate'),
array(0x0040,0x0251,'TM','1','PerformedProcedureStepEndTime'),
array(0x0040,0x0252,'CS','1','PerformedProcedureStepStatus'),
array(0x0040,0x0253,'CS','1','PerformedProcedureStepID'),
array(0x0040,0x0254,'LO','1','PerformedProcedureStepDescription'),
array(0x0040,0x0255,'LO','1','PerformedProcedureTypeDescription'),
array(0x0040,0x0260,'SQ','1','PerformedActionItemSequence'),
array(0x0040,0x0270,'SQ','1','ScheduledStepAttributesSequence'),
array(0x0040,0x0275,'SQ','1','RequestAttributesSequence'),
array(0x0040,0x0280,'ST','1','CommentsOnThePerformedProcedureSteps'),
array(0x0040,0x0293,'SQ','1','QuantitySequence'),
array(0x0040,0x0294,'DS','1','Quantity'),
array(0x0040,0x0295,'SQ','1','MeasuringUnitsSequence'),
array(0x0040,0x0296,'SQ','1','BillingItemSequence'),
array(0x0040,0x0300,'US','1','TotalTimeOfFluoroscopy'),
array(0x0040,0x0301,'US','1','TotalNumberOfExposures'),
array(0x0040,0x0302,'US','1','EntranceDose'),
array(0x0040,0x0303,'US','1-2','ExposedArea'),
array(0x0040,0x0306,'DS','1','DistanceSourceToEntrance'),
array(0x0040,0x0307,'DS','1','DistanceSourceToSupport'),
array(0x0040,0x0310,'ST','1','CommentsOnRadiationDose'),
array(0x0040,0x0312,'DS','1','XRayOutput'),
array(0x0040,0x0314,'DS','1','HalfValueLayer'),
array(0x0040,0x0316,'DS','1','OrganDose'),
array(0x0040,0x0318,'CS','1','OrganExposed'),
array(0x0040,0x0320,'SQ','1','BillingProcedureStepSequence'),
array(0x0040,0x0321,'SQ','1','FilmConsumptionSequence'),
array(0x0040,0x0324,'SQ','1','BillingSuppliesAndDevicesSequence'),
array(0x0040,0x0330,'SQ','1','ReferencedProcedureStepSequence'),
array(0x0040,0x0340,'SQ','1','PerformedSeriesSequence'),
array(0x0040,0x0400,'LT','1','CommentsOnScheduledProcedureStep'),
array(0x0040,0x050A,'LO','1','SpecimenAccessionNumber'),
array(0x0040,0x0550,'SQ','1','SpecimenSequence'),
array(0x0040,0x0551,'LO','1','SpecimenIdentifier'),
array(0x0040,0x0555,'SQ','1','AcquisitionContextSequence'),
array(0x0040,0x0556,'ST','1','AcquisitionContextDescription'),
array(0x0040,0x059A,'SQ','1','SpecimenTypeCodeSequence'),
array(0x0040,0x06FA,'LO','1','SlideIdentifier'),
array(0x0040,0x071A,'SQ','1','ImageCenterPointCoordinatesSequence'),
array(0x0040,0x072A,'DS','1','XOffsetInSlideCoordinateSystem'),
array(0x0040,0x073A,'DS','1','YOffsetInSlideCoordinateSystem'),
array(0x0040,0x074A,'DS','1','ZOffsetInSlideCoordinateSystem'),
array(0x0040,0x08D8,'SQ','1','PixelSpacingSequence'),
array(0x0040,0x08DA,'SQ','1','CoordinateSystemAxisCodeSequence'),
array(0x0040,0x08EA,'SQ','1','MeasurementUnitsCodeSequence'),
array(0x0040,0x1001,'SH','1','RequestedProcedureID'),
array(0x0040,0x1002,'LO','1','ReasonForRequestedProcedure'),
array(0x0040,0x1003,'SH','1','RequestedProcedurePriority'),
array(0x0040,0x1004,'LO','1','PatientTransportArrangements'),
array(0x0040,0x1005,'LO','1','RequestedProcedureLocation'),
array(0x0040,0x1006,'SH','1','PlacerOrderNumberOfProcedure'),
array(0x0040,0x1007,'SH','1','FillerOrderNumberOfProcedure'),
array(0x0040,0x1008,'LO','1','ConfidentialityCode'),
array(0x0040,0x1009,'SH','1','ReportingPriority'),
array(0x0040,0x1010,'PN','1-n','NamesOfIntendedRecipientsOfResults'),
array(0x0040,0x1400,'LT','1','RequestedProcedureComments'),
array(0x0040,0x2001,'LO','1','ReasonForTheImagingServiceRequest'),
array(0x0040,0x2002,'LO','1','ImagingServiceRequestDescription'),
array(0x0040,0x2004,'DA','1','IssueDateOfImagingServiceRequest'),
array(0x0040,0x2005,'TM','1','IssueTimeOfImagingServiceRequest'),
array(0x0040,0x2006,'SH','1','PlacerOrderNumberOfImagingServiceRequest'),
array(0x0040,0x2007,'SH','0','FillerOrderNumberOfImagingServiceRequest'),
array(0x0040,0x2008,'PN','1','OrderEnteredBy'),
array(0x0040,0x2009,'SH','1','OrderEntererLocation'),
array(0x0040,0x2010,'SH','1','OrderCallbackPhoneNumber'),
array(0x0040,0x2016,'LO','1','PlacerOrderNumberImagingServiceRequest'),
array(0x0040,0x2017,'LO','1','FillerOrderNumberImagingServiceRequest'),
array(0x0040,0x2400,'LT','1','ImagingServiceRequestComments'),
array(0x0040,0x3001,'LT','1','ConfidentialityConstraint'),
array(0x0040,0xA010,'CS','1','RelationshipType'),
array(0x0040,0xA027,'LO','1','VerifyingOrganization'),
array(0x0040,0xA030,'DT','1','VerificationDateTime'),
array(0x0040,0xA032,'DT','1','ObservationDateTime'),
array(0x0040,0xA040,'CS','1','ValueType'),
array(0x0040,0xA043,'SQ','1','ConceptNameCodeSequence'),
array(0x0040,0xA050,'CS','1','ContinuityOfContent'),
array(0x0040,0xA073,'SQ','1','VerifyingObserverSequence'),
array(0x0040,0xA075,'PN','1','VerifyingObserverName'),
array(0x0040,0xA088,'SQ','1','VerifyingObserverIdentificationCodeSeque'),
array(0x0040,0xA0B0,'US','2-2n','ReferencedWaveformChannels'),
array(0x0040,0xA120,'DT','1','DateTime'),
array(0x0040,0xA121,'DA','1','Date'),
array(0x0040,0xA122,'TM','1','Time'),
array(0x0040,0xA123,'PN','1','PersonName'),
array(0x0040,0xA124,'UI','1','UID'),
array(0x0040,0xA130,'CS','1','TemporalRangeType'),
array(0x0040,0xA132,'UL','1-n','ReferencedSamplePositionsU'),
array(0x0040,0xA136,'US','1-n','ReferencedFrameNumbers'),
array(0x0040,0xA138,'DS','1-n','ReferencedTimeOffsets'),
array(0x0040,0xA13A,'DT','1-n','ReferencedDatetime'),
array(0x0040,0xA160,'UT','1','TextValue'),
array(0x0040,0xA168,'SQ','1','ConceptCodeSequence'),
array(0x0040,0xA180,'US','1','AnnotationGroupNumber'),
array(0x0040,0xA195,'SQ','1','ConceptNameCodeSequenceModifier'),
array(0x0040,0xA300,'SQ','1','MeasuredValueSequence'),
array(0x0040,0xA30A,'DS','1-n','NumericValue'),
array(0x0040,0xA360,'SQ','1','PredecessorDocumentsSequence'),
array(0x0040,0xA370,'SQ','1','ReferencedRequestSequence'),
array(0x0040,0xA372,'SQ','1','PerformedProcedureCodeSequence'),
array(0x0040,0xA375,'SQ','1','CurrentRequestedProcedureEvidenceSequenSequence'),
array(0x0040,0xA385,'SQ','1','PertinentOtherEvidenceSequence'),
array(0x0040,0xA491,'CS','1','CompletionFlag'),
array(0x0040,0xA492,'LO','1','CompletionFlagDescription'),
array(0x0040,0xA493,'CS','1','VerificationFlag'),
array(0x0040,0xA504,'SQ','1','ContentTemplateSequence'),
array(0x0040,0xA525,'SQ','1','IdenticalDocumentsSequence'),
array(0x0040,0xA730,'SQ','1','ContentSequence'),
array(0x0040,0xB020,'SQ','1','AnnotationSequence'),
array(0x0040,0xDB00,'CS','1','TemplateIdentifier'),
array(0x0040,0xDB06,'DT','1','TemplateVersion'),
array(0x0040,0xDB07,'DT','1','TemplateLocalVersion'),
array(0x0040,0xDB0B,'CS','1','TemplateExtensionFlag'),
array(0x0040,0xDB0C,'UI','1','TemplateExtensionOrganizationUID'),
array(0x0040,0xDB0D,'UI','1','TemplateExtensionCreatorUID'),
array(0x0040,0xDB73,'UL','1-n','ReferencedContentItemIdentifier'),
array(0x0050,0x0000,'UL','1','XRayAngioDeviceGroupLength'),
array(0x0050,0x0004,'CS','1','CalibrationObject'),
array(0x0050,0x0010,'SQ','1','DeviceSequence'),
array(0x0050,0x0012,'CS','1','DeviceType'),
array(0x0050,0x0014,'DS','1','DeviceLength'),
array(0x0050,0x0016,'DS','1','DeviceDiameter'),
array(0x0050,0x0017,'CS','1','DeviceDiameterUnits'),
array(0x0050,0x0018,'DS','1','DeviceVolume'),
array(0x0050,0x0019,'DS','1','InterMarkerDistance'),
array(0x0050,0x0020,'LO','1','DeviceDescription'),
array(0x0050,0x0030,'SQ','1','CodedInterventionalDeviceSequence'),
array(0x0054,0x0000,'UL','1','NuclearMedicineGroupLength'),
array(0x0054,0x0010,'US','1-n','EnergyWindowVector'),
array(0x0054,0x0011,'US','1','NumberOfEnergyWindows'),
array(0x0054,0x0012,'SQ','1','EnergyWindowInformationSequence'),
array(0x0054,0x0013,'SQ','1','EnergyWindowRangeSequence'),
array(0x0054,0x0014,'DS','1','EnergyWindowLowerLimit'),
array(0x0054,0x0015,'DS','1','EnergyWindowUpperLimit'),
array(0x0054,0x0016,'SQ','1','RadiopharmaceuticalInformationSequence'),
array(0x0054,0x0017,'IS','1','ResidualSyringeCounts'),
array(0x0054,0x0018,'SH','1','EnergyWindowName'),
array(0x0054,0x0020,'US','1-n','DetectorVector'),
array(0x0054,0x0021,'US','1','NumberOfDetectors'),
array(0x0054,0x0022,'SQ','1','DetectorInformationSequence'),
array(0x0054,0x0030,'US','1-n','PhaseVector'),
array(0x0054,0x0031,'US','1','NumberOfPhases'),
array(0x0054,0x0032,'SQ','1','PhaseInformationSequence'),
array(0x0054,0x0033,'US','1','NumberOfFramesInPhase'),
array(0x0054,0x0036,'IS','1','PhaseDelay'),
array(0x0054,0x0038,'IS','1','PauseBetweenFrames'),
array(0x0054,0x0050,'US','1-n','RotationVector'),
array(0x0054,0x0051,'US','1','NumberOfRotations'),
array(0x0054,0x0052,'SQ','1','RotationInformationSequence'),
array(0x0054,0x0053,'US','1','NumberOfFramesInRotation'),
array(0x0054,0x0060,'US','1-n','RRIntervalVector'),
array(0x0054,0x0061,'US','1','NumberOfRRIntervals'),
array(0x0054,0x0062,'SQ','1','GatedInformationSequence'),
array(0x0054,0x0063,'SQ','1','DataInformationSequence'),
array(0x0054,0x0070,'US','1-n','TimeSlotVector'),
array(0x0054,0x0071,'US','1','NumberOfTimeSlots'),
array(0x0054,0x0072,'SQ','1','TimeSlotInformationSequence'),
array(0x0054,0x0073,'DS','1','TimeSlotTime'),
array(0x0054,0x0080,'US','1-n','SliceVector'),
array(0x0054,0x0081,'US','1','NumberOfSlices'),
array(0x0054,0x0090,'US','1-n','AngularViewVector'),
array(0x0054,0x0100,'US','1-n','TimeSliceVector'),
array(0x0054,0x0101,'US','1','NumberOfTimeSlices'),
array(0x0054,0x0200,'DS','1','StartAngle'),
array(0x0054,0x0202,'CS','1','TypeOfDetectorMotion'),
array(0x0054,0x0210,'IS','1-n','TriggerVector'),
array(0x0054,0x0211,'US','1','NumberOfTriggersInPhase'),
array(0x0054,0x0220,'SQ','1','ViewCodeSequence'),
array(0x0054,0x0222,'SQ','1','ViewAngulationModifierCodeSequence'),
array(0x0054,0x0300,'SQ','1','RadionuclideCodeSequence'),
array(0x0054,0x0302,'SQ','1','AdministrationRouteCodeSequence'),
array(0x0054,0x0304,'SQ','1','RadiopharmaceuticalCodeSequence'),
array(0x0054,0x0306,'SQ','1','CalibrationDataSequence'),
array(0x0054,0x0308,'US','1','EnergyWindowNumber'),
array(0x0054,0x0400,'SH','1','ImageID'),
array(0x0054,0x0410,'SQ','1','PatientOrientationCodeSequence'),
array(0x0054,0x0412,'SQ','1','PatientOrientationModifierCodeSequence'),
array(0x0054,0x0414,'SQ','1','PatientGantryRelationshipCodeSequence'),
array(0x0054,0x1000,'CS','2','SeriesType'),
array(0x0054,0x1001,'CS','1','Units'),
array(0x0054,0x1002,'CS','1','CountsSource'),
array(0x0054,0x1004,'CS','1','ReprojectionMethod'),
array(0x0054,0x1100,'CS','1','RandomsCorrectionMethod'),
array(0x0054,0x1101,'LO','1','AttenuationCorrectionMethod'),
array(0x0054,0x1102,'CS','1','DecayCorrection'),
array(0x0054,0x1103,'LO','1','ReconstructionMethod'),
array(0x0054,0x1104,'LO','1','DetectorLinesOfResponseUsed'),
array(0x0054,0x1105,'LO','1','ScatterCorrectionMethod'),
array(0x0054,0x1200,'DS','1','AxialAcceptance'),
array(0x0054,0x1201,'IS','2','AxialMash'),
array(0x0054,0x1202,'IS','1','TransverseMash'),
array(0x0054,0x1203,'DS','2','DetectorElementSize'),
array(0x0054,0x1210,'DS','1','CoincidenceWindowWidth'),
array(0x0054,0x1220,'CS','1-n','SecondaryCountsType'),
array(0x0054,0x1300,'DS','1','FrameReferenceTime'),
array(0x0054,0x1310,'IS','1','PrimaryPromptsCountsAccumulated'),
array(0x0054,0x1311,'IS','1-n','SecondaryCountsAccumulated'),
array(0x0054,0x1320,'DS','1','SliceSensitivityFactor'),
array(0x0054,0x1321,'DS','1','DecayFactor'),
array(0x0054,0x1322,'DS','1','DoseCalibrationFactor'),
array(0x0054,0x1323,'DS','1','ScatterFractionFactor'),
array(0x0054,0x1324,'DS','1','DeadTimeFactor'),
array(0x0054,0x1330,'US','1','ImageIndex'),
array(0x0054,0x1400,'CS','1-n','CountsIncluded'),
array(0x0054,0x1401,'CS','1','DeadTimeCorrectionFlag'),
array(0x0060,0x0000,'UL','1','HistogramGroupLength'),
array(0x0060,0x3000,'SQ','1','HistogramSequence'),
array(0x0060,0x3002,'US','1','HistogramNumberofBins'),
array(0x0060,0x3004,'US/SS','1','HistogramFirstBinValue'),
array(0x0060,0x3006,'US/SS','1','HistogramLastBinValue'),
array(0x0060,0x3008,'US','1','HistogramBinWidth'),
array(0x0060,0x3010,'LO','1','HistogramExplanation'),
array(0x0060,0x3020,'UL','1-n','HistogramData'),
array(0x0070,0x0001,'SQ','1','GraphicAnnotationSequence'),
array(0x0070,0x0002,'CS','1','GraphicLayer'),
array(0x0070,0x0003,'CS','1','BoundingBoxAnnotationUnits'),
array(0x0070,0x0004,'CS','1','AnchorPointAnnotationUnits'),
array(0x0070,0x0005,'CS','1','GraphicAnnotationUnits'),
array(0x0070,0x0006,'ST','1','UnformattedTextValue'),
array(0x0070,0x0008,'SQ','1','TextObjectSequence'),
array(0x0070,0x0009,'SQ','1','GraphicObjectSequence'),
array(0x0070,0x0010,'FL','2','BoundingBoxTopLeftHandCorner'),
array(0x0070,0x0011,'FL','2','BoundingBoxBottomRightHandCorner'),
array(0x0070,0x0012,'CS','1','BoundingBoxTextHorizontalJustification'),
array(0x0070,0x0014,'FL','2','AnchorPoint'),
array(0x0070,0x0015,'CS','1','AnchorPointVisibility'),
array(0x0070,0x0020,'US','1','GraphicDimensions'),
array(0x0070,0x0021,'US','1','NumberOfGraphicPoints'),
array(0x0070,0x0022,'FL','2-n','GraphicData'),
array(0x0070,0x0023,'CS','1','GraphicType'),
array(0x0070,0x0024,'CS','1','GraphicFilled'),
array(0x0070,0x0040,'IS','1','ImageRotationFrozenDraftRetired'),
array(0x0070,0x0041,'CS','1','ImageHorizontalFlip'),
array(0x0070,0x0042,'US','1','ImageRotation'),
array(0x0070,0x0050,'US','2','DisplayedAreaTLHCFrozenDraftRetired'),
array(0x0070,0x0051,'US','2','DisplayedAreaBRHCFrozenDraftRetired'),
array(0x0070,0x0052,'SL','2','DisplayedAreaTopLeftHandCorner'),
array(0x0070,0x0053,'SL','2','DisplayedAreaBottomRightHandCorner'),
array(0x0070,0x005A,'SQ','1','DisplayedAreaSelectionSequence'),
array(0x0070,0x0060,'SQ','1','GraphicLayerSequence'),
array(0x0070,0x0062,'IS','1','GraphicLayerOrder'),
array(0x0070,0x0066,'US','1','GraphicLayerRecommendedDisplayGrayscaleValue'),
array(0x0070,0x0067,'US','3','GraphicLayerRecommendedDisplayRGBValue'),
array(0x0070,0x0068,'LO','1','GraphicLayerDescription'),
array(0x0070,0x0080,'CS','1','PresentationLabel'),
array(0x0070,0x0081,'LO','1','PresentationDescription'),
array(0x0070,0x0082,'DA','1','PresentationCreationDate'),
array(0x0070,0x0083,'TM','1','PresentationCreationTime'),
array(0x0070,0x0084,'PN','1','PresentationCreatorsName'),
array(0x0070,0x0100,'CS','1','PresentationSizeMode'),
array(0x0070,0x0101,'DS','2','PresentationPixelSpacing'),
array(0x0070,0x0102,'IS','2','PresentationPixelAspectRatio'),
array(0x0070,0x0103,'FL','1','PresentationPixelMagnificationRatio'),
array(0x0088,0x0000,'UL','1','StorageGroupLength'),
array(0x0088,0x0130,'SH','1','StorageMediaFilesetID'),
array(0x0088,0x0140,'UI','1','StorageMediaFilesetUID'),
array(0x0088,0x0200,'SQ','1','IconImage'),
array(0x0088,0x0904,'LO','1','TopicTitle'),
array(0x0088,0x0906,'ST','1','TopicSubject'),
array(0x0088,0x0910,'LO','1','TopicAuthor'),
array(0x0088,0x0912,'LO','3','TopicKeyWords'),
array(0x1000,0x0000,'UL','1','CodeTableGroupLength'),
array(0x1000,0x0010,'US','3','EscapeTriplet'),
array(0x1000,0x0011,'US','3','RunLengthTriplet'),
array(0x1000,0x0012,'US','1','HuffmanTableSize'),
array(0x1000,0x0013,'US','3','HuffmanTableTriplet'),
array(0x1000,0x0014,'US','1','ShiftTableSize'),
array(0x1000,0x0015,'US','3','ShiftTableTriplet'),
array(0x1010,0x0000,'UL','1','ZonalMapGroupLength'),
array(0x1010,0x0004,'US','1-n','ZonalMap'),
array(0x2000,0x0000,'UL','1','FilmSessionGroupLength'),
array(0x2000,0x0010,'IS','1','NumberOfCopies'),
array(0x2000,0x001E,'SQ','1','PrinterConfigurationSequence'),
array(0x2000,0x0020,'CS','1','PrintPriority'),
array(0x2000,0x0030,'CS','1','MediumType'),
array(0x2000,0x0040,'CS','1','FilmDestination'),
array(0x2000,0x0050,'LO','1','FilmSessionLabel'),
array(0x2000,0x0060,'IS','1','MemoryAllocation'),
array(0x2000,0x0061,'IS','1','MaximumMemoryAllocation'),
array(0x2000,0x0062,'CS','1','ColorImagePrintingFlag'),
array(0x2000,0x0063,'CS','1','CollationFlag'),
array(0x2000,0x0065,'CS','1','AnnotationFlag'),
array(0x2000,0x0067,'CS','1','ImageOverlayFlag'),
array(0x2000,0x0069,'CS','1','PresentationLUTFlag'),
array(0x2000,0x006A,'CS','1','ImageBoxPresentationLUTFlag'),
array(0x2000,0x00A0,'US','1','MemoryBitDepth'),
array(0x2000,0x00A1,'US','1','PrintingBitDepth'),
array(0x2000,0x00A2,'SQ','1','MediaInstalledSequence'),
array(0x2000,0x00A4,'SQ','1','OtherMediaAvailableSequence'),
array(0x2000,0x00A8,'SQ','1','SupportedImageDisplayFormatsSequence'),
array(0x2000,0x0500,'SQ','1','ReferencedFilmBoxSequence'),
array(0x2000,0x0510,'SQ','1','ReferencedStoredPrintSequence'),
array(0x2010,0x0000,'UL','1','FilmBoxGroupLength'),
array(0x2010,0x0010,'ST','1','ImageDisplayFormat'),
array(0x2010,0x0030,'CS','1','AnnotationDisplayFormatID'),
array(0x2010,0x0040,'CS','1','FilmOrientation'),
array(0x2010,0x0050,'CS','1','FilmSizeID'),
array(0x2010,0x0052,'CS','1','PrinterResolutionID'),
array(0x2010,0x0054,'CS','1','DefaultPrinterResolutionID'),
array(0x2010,0x0060,'CS','1','MagnificationType'),
array(0x2010,0x0080,'CS','1','SmoothingType'),
array(0x2010,0x00A6,'CS','1','DefaultMagnificationType'),
array(0x2010,0x00A7,'CS','1-n','OtherMagnificationTypesAvailable'),
array(0x2010,0x00A8,'CS','1','DefaultSmoothingType'),
array(0x2010,0x00A9,'CS','1-n','OtherSmoothingTypesAvailable'),
array(0x2010,0x0100,'CS','1','BorderDensity'),
array(0x2010,0x0110,'CS','1','EmptyImageDensity'),
array(0x2010,0x0120,'US','1','MinDensity'),
array(0x2010,0x0130,'US','1','MaxDensity'),
array(0x2010,0x0140,'CS','1','Trim'),
array(0x2010,0x0150,'ST','1','ConfigurationInformation'),
array(0x2010,0x0152,'LT','1','ConfigurationInformationDescription'),
array(0x2010,0x0154,'IS','1','MaximumCollatedFilms'),
array(0x2010,0x015E,'US','1','Illumination'),
array(0x2010,0x0160,'US','1','ReflectedAmbientLight'),
array(0x2010,0x0376,'DS','2','PrinterPixelSpacing'),
array(0x2010,0x0500,'SQ','1','ReferencedFilmSessionSequence'),
array(0x2010,0x0510,'SQ','1','ReferencedImageBoxSequence'),
array(0x2010,0x0520,'SQ','1','ReferencedBasicAnnotationBoxSequence'),
array(0x2020,0x0000,'UL','1','ImageBoxGroupLength'),
array(0x2020,0x0010,'US','1','ImageBoxPosition'),
array(0x2020,0x0020,'CS','1','Polarity'),
array(0x2020,0x0030,'DS','1','RequestedImageSize'),
array(0x2020,0x0040,'CS','1','RequestedDecimateCropBehavior'),
array(0x2020,0x0050,'CS','1','RequestedResolutionID'),
array(0x2020,0x00A0,'CS','1','RequestedImageSizeFlag'),
array(0x2020,0x00A2,'CS','1','DecimateCropResult'),
array(0x2020,0x0110,'SQ','1','PreformattedGrayscaleImageSequence'),
array(0x2020,0x0111,'SQ','1','PreformattedColorImageSequence'),
array(0x2020,0x0130,'SQ','1','ReferencedImageOverlayBoxSequence'),
array(0x2020,0x0140,'SQ','1','ReferencedVOILUTBoxSequence'),
array(0x2030,0x0000,'UL','1','AnnotationGroupLength'),
array(0x2030,0x0010,'US','1','AnnotationPosition'),
array(0x2030,0x0020,'LO','1','TextString'),
array(0x2040,0x0000,'UL','1','OverlayBoxGroupLength'),
array(0x2040,0x0010,'SQ','1','ReferencedOverlayPlaneSequence'),
array(0x2040,0x0011,'US','9','ReferencedOverlayPlaneGroups'),
array(0x2040,0x0020,'SQ','1','OverlayPixelDataSequence'),
array(0x2040,0x0060,'CS','1','OverlayMagnificationType'),
array(0x2040,0x0070,'CS','1','OverlaySmoothingType'),
array(0x2040,0x0072,'CS','1','OverlayOrImageMagnification'),
array(0x2040,0x0074,'US','1','MagnifyToNumberOfColumns'),
array(0x2040,0x0080,'CS','1','OverlayForegroundDensity'),
array(0x2040,0x0082,'CS','1','OverlayBackgroundDensity'),
array(0x2040,0x0090,'CS','1','OverlayMode'),
array(0x2040,0x0100,'CS','1','ThresholdDensity'),
array(0x2040,0x0500,'SQ','1','ReferencedOverlayImageBoxSequence'),
array(0x2050,0x0000,'UL','1','PresentationLUTGroupLength'),
array(0x2050,0x0010,'SQ','1','PresentationLUTSequence'),
array(0x2050,0x0020,'CS','1','PresentationLUTShape'),
array(0x2050,0x0500,'SQ','1','ReferencedPresentationLUTSequence'),
array(0x2100,0x0000,'UL','1','PrintJobGroupLength'),
array(0x2100,0x0010,'SH','1','PrintJobID'),
array(0x2100,0x0020,'CS','1','ExecutionStatus'),
array(0x2100,0x0030,'CS','1','ExecutionStatusInfo'),
array(0x2100,0x0040,'DA','1','CreationDate'),
array(0x2100,0x0050,'TM','1','CreationTime'),
array(0x2100,0x0070,'AE','1','Originator'),
array(0x2100,0x0140,'AE','1','DestinationAE'),
array(0x2100,0x0160,'SH','1','OwnerID'),
array(0x2100,0x0170,'IS','1','NumberOfFilms'),
array(0x2100,0x0500,'SQ','1','ReferencedPrintJobSequence'),
array(0x2110,0x0000,'UL','1','PrinterGroupLength'),
array(0x2110,0x0010,'CS','1','PrinterStatus'),
array(0x2110,0x0020,'CS','1','PrinterStatusInfo'),
array(0x2110,0x0030,'LO','1','PrinterName'),
array(0x2110,0x0099,'SH','1','PrintQueueID'),
array(0x2120,0x0000,'UL','1','QueueGroupLength'),
array(0x2120,0x0010,'CS','1','QueueStatus'),
array(0x2120,0x0050,'SQ','1','PrintJobDescriptionSequence'),
array(0x2120,0x0070,'SQ','1','QueueReferencedPrintJobSequence'),
array(0x2130,0x0000,'UL','1','PrintContentGroupLength'),
array(0x2130,0x0010,'SQ','1','PrintManagementCapabilitiesSequence'),
array(0x2130,0x0015,'SQ','1','PrinterCharacteristicsSequence'),
array(0x2130,0x0030,'SQ','1','FilmBoxContentSequence'),
array(0x2130,0x0040,'SQ','1','ImageBoxContentSequence'),
array(0x2130,0x0050,'SQ','1','AnnotationContentSequence'),
array(0x2130,0x0060,'SQ','1','ImageOverlayBoxContentSequence'),
array(0x2130,0x0080,'SQ','1','PresentationLUTContentSequence'),
array(0x2130,0x00A0,'SQ','1','ProposedStudySequence'),
array(0x2130,0x00C0,'SQ','1','OriginalImageSequence'),
array(0x3002,0x0000,'UL','1','RTImageGroupLength'),
array(0x3002,0x0002,'SH','1','RTImageLabel'),
array(0x3002,0x0003,'LO','1','RTImageName'),
array(0x3002,0x0004,'ST','1','RTImageDescription'),
array(0x3002,0x000A,'CS','1','ReportedValuesOrigin'),
array(0x3002,0x000C,'CS','1','RTImagePlane'),
array(0x3002,0x000D,'DS','3','XRayImageReceptorTranslation'),
array(0x3002,0x000E,'DS','1','XRayImageReceptorAngle'),
array(0x3002,0x0010,'DS','6','RTImageOrientation'),
array(0x3002,0x0011,'DS','2','ImagePlanePixelSpacing'),
array(0x3002,0x0012,'DS','2','RTImagePosition'),
array(0x3002,0x0020,'SH','1','RadiationMachineName'),
array(0x3002,0x0022,'DS','1','RadiationMachineSAD'),
array(0x3002,0x0024,'DS','1','RadiationMachineSSD'),
array(0x3002,0x0026,'DS','1','RTImageSID'),
array(0x3002,0x0028,'DS','1','SourceToReferenceObjectDistance'),
array(0x3002,0x0029,'IS','1','FractionNumber'),
array(0x3002,0x0030,'SQ','1','ExposureSequence'),
array(0x3002,0x0032,'DS','1','MetersetExposure'),
array(0x3004,0x0000,'UL','1','RTDoseGroupLength'),
array(0x3004,0x0001,'CS','1','DVHType'),
array(0x3004,0x0002,'CS','1','DoseUnits'),
array(0x3004,0x0004,'CS','1','DoseType'),
array(0x3004,0x0006,'LO','1','DoseComment'),
array(0x3004,0x0008,'DS','3','NormalizationPoint'),
array(0x3004,0x000A,'CS','1','DoseSummationType'),
array(0x3004,0x000C,'DS','2-n','GridFrameOffsetVector'),
array(0x3004,0x000E,'DS','1','DoseGridScaling'),
array(0x3004,0x0010,'SQ','1','RTDoseROISequence'),
array(0x3004,0x0012,'DS','1','DoseValue'),
array(0x3004,0x0040,'DS','3','DVHNormalizationPoint'),
array(0x3004,0x0042,'DS','1','DVHNormalizationDoseValue'),
array(0x3004,0x0050,'SQ','1','DVHSequence'),
array(0x3004,0x0052,'DS','1','DVHDoseScaling'),
array(0x3004,0x0054,'CS','1','DVHVolumeUnits'),
array(0x3004,0x0056,'IS','1','DVHNumberOfBins'),
array(0x3004,0x0058,'DS','2-2n','DVHData'),
array(0x3004,0x0060,'SQ','1','DVHReferencedROISequence'),
array(0x3004,0x0062,'CS','1','DVHROIContributionType'),
array(0x3004,0x0070,'DS','1','DVHMinimumDose'),
array(0x3004,0x0072,'DS','1','DVHMaximumDose'),
array(0x3004,0x0074,'DS','1','DVHMeanDose'),
array(0x3006,0x0000,'UL','1','RTStructureSetGroupLength'),
array(0x3006,0x0002,'SH','1','StructureSetLabel'),
array(0x3006,0x0004,'LO','1','StructureSetName'),
array(0x3006,0x0006,'ST','1','StructureSetDescription'),
array(0x3006,0x0008,'DA','1','StructureSetDate'),
array(0x3006,0x0009,'TM','1','StructureSetTime'),
array(0x3006,0x0010,'SQ','1','ReferencedFrameOfReferenceSequence'),
array(0x3006,0x0012,'SQ','1','RTReferencedStudySequence'),
array(0x3006,0x0014,'SQ','1','RTReferencedSeriesSequence'),
array(0x3006,0x0016,'SQ','1','ContourImageSequence'),
array(0x3006,0x0020,'SQ','1','StructureSetROISequence'),
array(0x3006,0x0022,'IS','1','ROINumber'),
array(0x3006,0x0024,'UI','1','ReferencedFrameOfReferenceUID'),
array(0x3006,0x0026,'LO','1','ROIName'),
array(0x3006,0x0028,'ST','1','ROIDescription'),
array(0x3006,0x002A,'IS','3','ROIDisplayColor'),
array(0x3006,0x002C,'DS','1','ROIVolume'),
array(0x3006,0x0030,'SQ','1','RTRelatedROISequence'),
array(0x3006,0x0033,'CS','1','RTROIRelationship'),
array(0x3006,0x0036,'CS','1','ROIGenerationAlgorithm'),
array(0x3006,0x0038,'LO','1','ROIGenerationDescription'),
array(0x3006,0x0039,'SQ','1','ROIContourSequence'),
array(0x3006,0x0040,'SQ','1','ContourSequence'),
array(0x3006,0x0042,'CS','1','ContourGeometricType'),
array(0x3006,0x0044,'DS','1','ContourSlabThickness'),
array(0x3006,0x0045,'DS','3','ContourOffsetVector'),
array(0x3006,0x0046,'IS','1','NumberOfContourPoints'),
array(0x3006,0x0048,'IS','1','ContourNumber'),
array(0x3006,0x0049,'IS','1-n','AttachedContours'),
array(0x3006,0x0050,'DS','3-3n','ContourData'),
array(0x3006,0x0080,'SQ','1','RTROIObservationsSequence'),
array(0x3006,0x0082,'IS','1','ObservationNumber'),
array(0x3006,0x0084,'IS','1','ReferencedROINumber'),
array(0x3006,0x0085,'SH','1','ROIObservationLabel'),
array(0x3006,0x0086,'SQ','1','RTROIIdentificationCodeSequence'),
array(0x3006,0x0088,'ST','1','ROIObservationDescription'),
array(0x3006,0x00A0,'SQ','1','RelatedRTROIObservationsSequence'),
array(0x3006,0x00A4,'CS','1','RTROIInterpretedType'),
array(0x3006,0x00A6,'PN','1','ROIInterpreter'),
array(0x3006,0x00B0,'SQ','1','ROIPhysicalPropertiesSequence'),
array(0x3006,0x00B2,'CS','1','ROIPhysicalProperty'),
array(0x3006,0x00B4,'DS','1','ROIPhysicalPropertyValue'),
array(0x3006,0x00C0,'SQ','1','FrameOfReferenceRelationshipSequence'),
array(0x3006,0x00C2,'UI','1','RelatedFrameOfReferenceUID'),
array(0x3006,0x00C4,'CS','1','FrameOfReferenceTransformationType'),
array(0x3006,0x00C6,'DS','16','FrameOfReferenceTransformationMatrix'),
array(0x3006,0x00C8,'LO','1','FrameOfReferenceTransformationComment'),
array(0x3008,0x0010,'SQ','1','MeasuredDoseReferenceSequence'),
array(0x3008,0x0012,'ST','1','MeasuredDoseDescription'),
array(0x3008,0x0014,'CS','1','MeasuredDoseType'),
array(0x3008,0x0016,'DS','1','MeasuredDoseValue'),
array(0x3008,0x0020,'SQ','1','TreatmentSessionBeamSequence'),
array(0x3008,0x0022,'IS','1','CurrentFractionNumber'),
array(0x3008,0x0024,'DA','1','TreatmentControlPointDate'),
array(0x3008,0x0025,'TM','1','TreatmentControlPointTime'),
array(0x3008,0x002A,'CS','1','TreatmentTerminationStatus'),
array(0x3008,0x002B,'SH','1','TreatmentTerminationCode'),
array(0x3008,0x002C,'CS','1','TreatmentVerificationStatus'),
array(0x3008,0x0030,'SQ','1','ReferencedTreatmentRecordSequence'),
array(0x3008,0x0032,'DS','1','SpecifiedPrimaryMeterset'),
array(0x3008,0x0033,'DS','1','SpecifiedSecondaryMeterset'),
array(0x3008,0x0036,'DS','1','DeliveredPrimaryMeterset'),
array(0x3008,0x0037,'DS','1','DeliveredSecondaryMeterset'),
array(0x3008,0x003A,'DS','1','SpecifiedTreatmentTime'),
array(0x3008,0x003B,'DS','1','DeliveredTreatmentTime'),
array(0x3008,0x0040,'SQ','1','ControlPointDeliverySequence'),
array(0x3008,0x0042,'DS','1','SpecifiedMeterset'),
array(0x3008,0x0044,'DS','1','DeliveredMeterset'),
array(0x3008,0x0048,'DS','1','DoseRateDelivered'),
array(0x3008,0x0050,'SQ','1','TreatmentSummaryCalculatedDoseReferenceSequence'),
array(0x3008,0x0052,'DS','1','CumulativeDosetoDoseReference'),
array(0x3008,0x0054,'DA','1','FirstTreatmentDate'),
array(0x3008,0x0056,'DA','1','MostRecentTreatmentDate'),
array(0x3008,0x005A,'IS','1','NumberofFractionsDelivered'),
array(0x3008,0x0060,'SQ','1','OverrideSequence'),
array(0x3008,0x0062,'AT','1','OverrideParameterPointer'),
array(0x3008,0x0064,'IS','1','MeasuredDoseReferenceNumber'),
array(0x3008,0x0066,'ST','1','OverrideReason'),
array(0x3008,0x0070,'SQ','1','CalculatedDoseReferenceSequence'),
array(0x3008,0x0072,'IS','1','CalculatedDoseReferenceNumber'),
array(0x3008,0x0074,'ST','1','CalculatedDoseReferenceDescription'),
array(0x3008,0x0076,'DS','1','CalculatedDoseReferenceDoseValue'),
array(0x3008,0x0078,'DS','1','StartMeterset'),
array(0x3008,0x007A,'DS','1','EndMeterset'),
array(0x3008,0x0080,'SQ','1','ReferencedMeasuredDoseReferenceSequence'),
array(0x3008,0x0082,'IS','1','ReferencedMeasuredDoseReferenceNumber'),
array(0x3008,0x0090,'SQ','1','ReferencedCalculatedDoseReferenceSequence'),
array(0x3008,0x0092,'IS','1','ReferencedCalculatedDoseReferenceNumber'),
array(0x3008,0x00A0,'SQ','1','BeamLimitingDeviceLeafPairsSequence'),
array(0x3008,0x00B0,'SQ','1','RecordedWedgeSequence'),
array(0x3008,0x00C0,'SQ','1','RecordedCompensatorSequence'),
array(0x3008,0x00D0,'SQ','1','RecordedBlockSequence'),
array(0x3008,0x00E0,'SQ','1','TreatmentSummaryMeasuredDoseReferenceSequence'),
array(0x3008,0x0100,'SQ','1','RecordedSourceSequence'),
array(0x3008,0x0105,'LO','1','SourceSerialNumber'),
array(0x3008,0x0110,'SQ','1','TreatmentSessionApplicationSetupSequence'),
array(0x3008,0x0116,'CS','1','ApplicationSetupCheck'),
array(0x3008,0x0120,'SQ','1','RecordedBrachyAccessoryDeviceSequence'),
array(0x3008,0x0122,'IS','1','ReferencedBrachyAccessoryDeviceNumber'),
array(0x3008,0x0130,'SQ','1','RecordedChannelSequence'),
array(0x3008,0x0132,'DS','1','SpecifiedChannelTotalTime'),
array(0x3008,0x0134,'DS','1','DeliveredChannelTotalTime'),
array(0x3008,0x0136,'IS','1','SpecifiedNumberofPulses'),
array(0x3008,0x0138,'IS','1','DeliveredNumberofPulses'),
array(0x3008,0x013A,'DS','1','SpecifiedPulseRepetitionInterval'),
array(0x3008,0x013C,'DS','1','DeliveredPulseRepetitionInterval'),
array(0x3008,0x0140,'SQ','1','RecordedSourceApplicatorSequence'),
array(0x3008,0x0142,'IS','1','ReferencedSourceApplicatorNumber'),
array(0x3008,0x0150,'SQ','1','RecordedChannelShieldSequence'),
array(0x3008,0x0152,'IS','1','ReferencedChannelShieldNumber'),
array(0x3008,0x0160,'SQ','1','BrachyControlPointDeliveredSequence'),
array(0x3008,0x0162,'DA','1','SafePositionExitDate'),
array(0x3008,0x0164,'TM','1','SafePositionExitTime'),
array(0x3008,0x0166,'DA','1','SafePositionReturnDate'),
array(0x3008,0x0168,'TM','1','SafePositionReturnTime'),
array(0x3008,0x0200,'CS','1','CurrentTreatmentStatus'),
array(0x3008,0x0202,'ST','1','TreatmentStatusComment'),
array(0x3008,0x0220,'SQ','1','FractionGroupSummarySequence'),
array(0x3008,0x0223,'IS','1','ReferencedFractionNumber'),
array(0x3008,0x0224,'CS','1','FractionGroupType'),
array(0x3008,0x0230,'CS','1','BeamStopperPosition'),
array(0x3008,0x0240,'SQ','1','FractionStatusSummarySequence'),
array(0x3008,0x0250,'DA','1','TreatmentDate'),
array(0x3008,0x0251,'TM','1','TreatmentTime'),
array(0x300A,0x0000,'UL','1','RTPlanGroupLength'),
array(0x300A,0x0002,'SH','1','RTPlanLabel'),
array(0x300A,0x0003,'LO','1','RTPlanName'),
array(0x300A,0x0004,'ST','1','RTPlanDescription'),
array(0x300A,0x0006,'DA','1','RTPlanDate'),
array(0x300A,0x0007,'TM','1','RTPlanTime'),
array(0x300A,0x0009,'LO','1-n','TreatmentProtocols'),
array(0x300A,0x000A,'CS','1','TreatmentIntent'),
array(0x300A,0x000B,'LO','1-n','TreatmentSites'),
array(0x300A,0x000C,'CS','1','RTPlanGeometry'),
array(0x300A,0x000E,'ST','1','PrescriptionDescription'),
array(0x300A,0x0010,'SQ','1','DoseReferenceSequence'),
array(0x300A,0x0012,'IS','1','DoseReferenceNumber'),
array(0x300A,0x0014,'CS','1','DoseReferenceStructureType'),
array(0x300A,0x0015,'CS','1','NominalBeamEnergyUnit'),
array(0x300A,0x0016,'LO','1','DoseReferenceDescription'),
array(0x300A,0x0018,'DS','3','DoseReferencePointCoordinates'),
array(0x300A,0x001A,'DS','1','NominalPriorDose'),
array(0x300A,0x0020,'CS','1','DoseReferenceType'),
array(0x300A,0x0021,'DS','1','ConstraintWeight'),
array(0x300A,0x0022,'DS','1','DeliveryWarningDose'),
array(0x300A,0x0023,'DS','1','DeliveryMaximumDose'),
array(0x300A,0x0025,'DS','1','TargetMinimumDose'),
array(0x300A,0x0026,'DS','1','TargetPrescriptionDose'),
array(0x300A,0x0027,'DS','1','TargetMaximumDose'),
array(0x300A,0x0028,'DS','1','TargetUnderdoseVolumeFraction'),
array(0x300A,0x002A,'DS','1','OrganAtRiskFullVolumeDose'),
array(0x300A,0x002B,'DS','1','OrganAtRiskLimitDose'),
array(0x300A,0x002C,'DS','1','OrganAtRiskMaximumDose'),
array(0x300A,0x002D,'DS','1','OrganAtRiskOverdoseVolumeFraction'),
array(0x300A,0x0040,'SQ','1','ToleranceTableSequence'),
array(0x300A,0x0042,'IS','1','ToleranceTableNumber'),
array(0x300A,0x0043,'SH','1','ToleranceTableLabel'),
array(0x300A,0x0044,'DS','1','GantryAngleTolerance'),
array(0x300A,0x0046,'DS','1','BeamLimitingDeviceAngleTolerance'),
array(0x300A,0x0048,'SQ','1','BeamLimitingDeviceToleranceSequence'),
array(0x300A,0x004A,'DS','1','BeamLimitingDevicePositionTolerance'),
array(0x300A,0x004C,'DS','1','PatientSupportAngleTolerance'),
array(0x300A,0x004E,'DS','1','TableTopEccentricAngleTolerance'),
array(0x300A,0x0051,'DS','1','TableTopVerticalPositionTolerance'),
array(0x300A,0x0052,'DS','1','TableTopLongitudinalPositionTolerance'),
array(0x300A,0x0053,'DS','1','TableTopLateralPositionTolerance'),
array(0x300A,0x0055,'CS','1','RTPlanRelationship'),
array(0x300A,0x0070,'SQ','1','FractionGroupSequence'),
array(0x300A,0x0071,'IS','1','FractionGroupNumber'),
array(0x300A,0x0078,'IS','1','NumberOfFractionsPlanned'),
array(0x300A,0x0079,'IS','1','NumberOfFractionsPerDay'),
array(0x300A,0x007A,'IS','1','RepeatFractionCycleLength'),
array(0x300A,0x007B,'LT','1','FractionPattern'),
array(0x300A,0x0080,'IS','1','NumberOfBeams'),
array(0x300A,0x0082,'DS','3','BeamDoseSpecificationPoint'),
array(0x300A,0x0084,'DS','1','BeamDose'),
array(0x300A,0x0086,'DS','1','BeamMeterset'),
array(0x300A,0x00A0,'IS','1','NumberOfBrachyApplicationSetups'),
array(0x300A,0x00A2,'DS','3','BrachyApplicationSetupDoseSpecificationPoint'),
array(0x300A,0x00A4,'DS','1','BrachyApplicationSetupDose'),
array(0x300A,0x00B0,'SQ','1','BeamSequence'),
array(0x300A,0x00B2,'SH','1','TreatmentMachineName'),
array(0x300A,0x00B3,'CS','1','PrimaryDosimeterUnit'),
array(0x300A,0x00B4,'DS','1','SourceAxisDistance'),
array(0x300A,0x00B6,'SQ','1','BeamLimitingDeviceSequence'),
array(0x300A,0x00B8,'CS','1','RTBeamLimitingDeviceType'),
array(0x300A,0x00BA,'DS','1','SourceToBeamLimitingDeviceDistance'),
array(0x300A,0x00BC,'IS','1','NumberOfLeafJawPairs'),
array(0x300A,0x00BE,'DS','3-n','LeafPositionBoundaries'),
array(0x300A,0x00C0,'IS','1','BeamNumber'),
array(0x300A,0x00C2,'LO','1','BeamName'),
array(0x300A,0x00C3,'ST','1','BeamDescription'),
array(0x300A,0x00C4,'CS','1','BeamType'),
array(0x300A,0x00C6,'CS','1','RadiationType'),
array(0x300A,0x00C8,'IS','1','ReferenceImageNumber'),
array(0x300A,0x00CA,'SQ','1','PlannedVerificationImageSequence'),
array(0x300A,0x00CC,'LO','1-n','ImagingDeviceSpecificAcquisitionParameters'),
array(0x300A,0x00CE,'CS','1','TreatmentDeliveryType'),
array(0x300A,0x00D0,'IS','1','NumberOfWedges'),
array(0x300A,0x00D1,'SQ','1','WedgeSequence'),
array(0x300A,0x00D2,'IS','1','WedgeNumber'),
array(0x300A,0x00D3,'CS','1','WedgeType'),
array(0x300A,0x00D4,'SH','1','WedgeID'),
array(0x300A,0x00D5,'IS','1','WedgeAngle'),
array(0x300A,0x00D6,'DS','1','WedgeFactor'),
array(0x300A,0x00D8,'DS','1','WedgeOrientation'),
array(0x300A,0x00DA,'DS','1','SourceToWedgeTrayDistance'),
array(0x300A,0x00E0,'IS','1','NumberOfCompensators'),
array(0x300A,0x00E1,'SH','1','MaterialID'),
array(0x300A,0x00E2,'DS','1','TotalCompensatorTrayFactor'),
array(0x300A,0x00E3,'SQ','1','CompensatorSequence'),
array(0x300A,0x00E4,'IS','1','CompensatorNumber'),
array(0x300A,0x00E5,'SH','1','CompensatorID'),
array(0x300A,0x00E6,'DS','1','SourceToCompensatorTrayDistance'),
array(0x300A,0x00E7,'IS','1','CompensatorRows'),
array(0x300A,0x00E8,'IS','1','CompensatorColumns'),
array(0x300A,0x00E9,'DS','2','CompensatorPixelSpacing'),
array(0x300A,0x00EA,'DS','2','CompensatorPosition'),
array(0x300A,0x00EB,'DS','1-n','CompensatorTransmissionData'),
array(0x300A,0x00EC,'DS','1-n','CompensatorThicknessData'),
array(0x300A,0x00ED,'IS','1','NumberOfBoli'),
array(0x300A,0x00EE,'CS','1','CompensatorType'),
array(0x300A,0x00F0,'IS','1','NumberOfBlocks'),
array(0x300A,0x00F2,'DS','1','TotalBlockTrayFactor'),
array(0x300A,0x00F4,'SQ','1','BlockSequence'),
array(0x300A,0x00F5,'SH','1','BlockTrayID'),
array(0x300A,0x00F6,'DS','1','SourceToBlockTrayDistance'),
array(0x300A,0x00F8,'CS','1','BlockType'),
array(0x300A,0x00FA,'CS','1','BlockDivergence'),
array(0x300A,0x00FC,'IS','1','BlockNumber'),
array(0x300A,0x00FE,'LO','1','BlockName'),
array(0x300A,0x0100,'DS','1','BlockThickness'),
array(0x300A,0x0102,'DS','1','BlockTransmission'),
array(0x300A,0x0104,'IS','1','BlockNumberOfPoints'),
array(0x300A,0x0106,'DS','2-2n','BlockData'),
array(0x300A,0x0107,'SQ','1','ApplicatorSequence'),
array(0x300A,0x0108,'SH','1','ApplicatorID'),
array(0x300A,0x0109,'CS','1','ApplicatorType'),
array(0x300A,0x010A,'LO','1','ApplicatorDescription'),
array(0x300A,0x010C,'DS','1','CumulativeDoseReferenceCoefficient'),
array(0x300A,0x010E,'DS','1','FinalCumulativeMetersetWeight'),
array(0x300A,0x0110,'IS','1','NumberOfControlPoints'),
array(0x300A,0x0111,'SQ','1','ControlPointSequence'),
array(0x300A,0x0112,'IS','1','ControlPointIndex'),
array(0x300A,0x0114,'DS','1','NominalBeamEnergy'),
array(0x300A,0x0115,'DS','1','DoseRateSet'),
array(0x300A,0x0116,'SQ','1','WedgePositionSequence'),
array(0x300A,0x0118,'CS','1','WedgePosition'),
array(0x300A,0x011A,'SQ','1','BeamLimitingDevicePositionSequence'),
array(0x300A,0x011C,'DS','2-2n','LeafJawPositions'),
array(0x300A,0x011E,'DS','1','GantryAngle'),
array(0x300A,0x011F,'CS','1','GantryRotationDirection'),
array(0x300A,0x0120,'DS','1','BeamLimitingDeviceAngle'),
array(0x300A,0x0121,'CS','1','BeamLimitingDeviceRotationDirection'),
array(0x300A,0x0122,'DS','1','PatientSupportAngle'),
array(0x300A,0x0123,'CS','1','PatientSupportRotationDirection'),
array(0x300A,0x0124,'DS','1','TableTopEccentricAxisDistance'),
array(0x300A,0x0125,'DS','1','TableTopEccentricAngle'),
array(0x300A,0x0126,'CS','1','TableTopEccentricRotationDirection'),
array(0x300A,0x0128,'DS','1','TableTopVerticalPosition'),
array(0x300A,0x0129,'DS','1','TableTopLongitudinalPosition'),
array(0x300A,0x012A,'DS','1','TableTopLateralPosition'),
array(0x300A,0x012C,'DS','3','IsocenterPosition'),
array(0x300A,0x012E,'DS','3','SurfaceEntryPoint'),
array(0x300A,0x0130,'DS','1','SourceToSurfaceDistance'),
array(0x300A,0x0134,'DS','1','CumulativeMetersetWeight'),
array(0x300A,0x0180,'SQ','1','PatientSetupSequence'),
array(0x300A,0x0182,'IS','1','PatientSetupNumber'),
array(0x300A,0x0184,'LO','1','PatientAdditionalPosition'),
array(0x300A,0x0190,'SQ','1','FixationDeviceSequence'),
array(0x300A,0x0192,'CS','1','FixationDeviceType'),
array(0x300A,0x0194,'SH','1','FixationDeviceLabel'),
array(0x300A,0x0196,'ST','1','FixationDeviceDescription'),
array(0x300A,0x0198,'SH','1','FixationDevicePosition'),
array(0x300A,0x01A0,'SQ','1','ShieldingDeviceSequence'),
array(0x300A,0x01A2,'CS','1','ShieldingDeviceType'),
array(0x300A,0x01A4,'SH','1','ShieldingDeviceLabel'),
array(0x300A,0x01A6,'ST','1','ShieldingDeviceDescription'),
array(0x300A,0x01A8,'SH','1','ShieldingDevicePosition'),
array(0x300A,0x01B0,'CS','1','SetupTechnique'),
array(0x300A,0x01B2,'ST','1','SetupTechniqueDescription'),
array(0x300A,0x01B4,'SQ','1','SetupDeviceSequence'),
array(0x300A,0x01B6,'CS','1','SetupDeviceType'),
array(0x300A,0x01B8,'SH','1','SetupDeviceLabel'),
array(0x300A,0x01BA,'ST','1','SetupDeviceDescription'),
array(0x300A,0x01BC,'DS','1','SetupDeviceParameter'),
array(0x300A,0x01D0,'ST','1','SetupReferenceDescription'),
array(0x300A,0x01D2,'DS','1','TableTopVerticalSetupDisplacement'),
array(0x300A,0x01D4,'DS','1','TableTopLongitudinalSetupDisplacement'),
array(0x300A,0x01D6,'DS','1','TableTopLateralSetupDisplacement'),
array(0x300A,0x0200,'CS','1','BrachyTreatmentTechnique'),
array(0x300A,0x0202,'CS','1','BrachyTreatmentType'),
array(0x300A,0x0206,'SQ','1','TreatmentMachineSequence'),
array(0x300A,0x0210,'SQ','1','SourceSequence'),
array(0x300A,0x0212,'IS','1','SourceNumber'),
array(0x300A,0x0214,'CS','1','SourceType'),
array(0x300A,0x0216,'LO','1','SourceManufacturer'),
array(0x300A,0x0218,'DS','1','ActiveSourceDiameter'),
array(0x300A,0x021A,'DS','1','ActiveSourceLength'),
array(0x300A,0x0222,'DS','1','SourceEncapsulationNominalThickness'),
array(0x300A,0x0224,'DS','1','SourceEncapsulationNominalTransmission'),
array(0x300A,0x0226,'LO','1','SourceIsotopeName'),
array(0x300A,0x0228,'DS','1','SourceIsotopeHalfLife'),
array(0x300A,0x022A,'DS','1','ReferenceAirKermaRate'),
array(0x300A,0x022C,'DA','1','AirKermaRateReferenceDate'),
array(0x300A,0x022E,'TM','1','AirKermaRateReferenceTime'),
array(0x300A,0x0230,'SQ','1','ApplicationSetupSequence'),
array(0x300A,0x0232,'CS','1','ApplicationSetupType'),
array(0x300A,0x0234,'IS','1','ApplicationSetupNumber'),
array(0x300A,0x0236,'LO','1','ApplicationSetupName'),
array(0x300A,0x0238,'LO','1','ApplicationSetupManufacturer'),
array(0x300A,0x0240,'IS','1','TemplateNumber'),
array(0x300A,0x0242,'SH','1','TemplateType'),
array(0x300A,0x0244,'LO','1','TemplateName'),
array(0x300A,0x0250,'DS','1','TotalReferenceAirKerma'),
array(0x300A,0x0260,'SQ','1','BrachyAccessoryDeviceSequence'),
array(0x300A,0x0262,'IS','1','BrachyAccessoryDeviceNumber'),
array(0x300A,0x0263,'SH','1','BrachyAccessoryDeviceID'),
array(0x300A,0x0264,'CS','1','BrachyAccessoryDeviceType'),
array(0x300A,0x0266,'LO','1','BrachyAccessoryDeviceName'),
array(0x300A,0x026A,'DS','1','BrachyAccessoryDeviceNominalThickness'),
array(0x300A,0x026C,'DS','1','BrachyAccessoryDeviceNominalTransmission'),
array(0x300A,0x0280,'SQ','1','ChannelSequence'),
array(0x300A,0x0282,'IS','1','ChannelNumber'),
array(0x300A,0x0284,'DS','1','ChannelLength'),
array(0x300A,0x0286,'DS','1','ChannelTotalTime'),
array(0x300A,0x0288,'CS','1','SourceMovementType'),
array(0x300A,0x028A,'IS','1','NumberOfPulses'),
array(0x300A,0x028C,'DS','1','PulseRepetitionInterval'),
array(0x300A,0x0290,'IS','1','SourceApplicatorNumber'),
array(0x300A,0x0291,'SH','1','SourceApplicatorID'),
array(0x300A,0x0292,'CS','1','SourceApplicatorType'),
array(0x300A,0x0294,'LO','1','SourceApplicatorName'),
array(0x300A,0x0296,'DS','1','SourceApplicatorLength'),
array(0x300A,0x0298,'LO','1','SourceApplicatorManufacturer'),
array(0x300A,0x029C,'DS','1','SourceApplicatorWallNominalThickness'),
array(0x300A,0x029E,'DS','1','SourceApplicatorWallNominalTransmission'),
array(0x300A,0x02A0,'DS','1','SourceApplicatorStepSize'),
array(0x300A,0x02A2,'IS','1','TransferTubeNumber'),
array(0x300A,0x02A4,'DS','1','TransferTubeLength'),
array(0x300A,0x02B0,'SQ','1','ChannelShieldSequence'),
array(0x300A,0x02B2,'IS','1','ChannelShieldNumber'),
array(0x300A,0x02B3,'SH','1','ChannelShieldID'),
array(0x300A,0x02B4,'LO','1','ChannelShieldName'),
array(0x300A,0x02B8,'DS','1','ChannelShieldNominalThickness'),
array(0x300A,0x02BA,'DS','1','ChannelShieldNominalTransmission'),
array(0x300A,0x02C8,'DS','1','FinalCumulativeTimeWeight'),
array(0x300A,0x02D0,'SQ','1','BrachyControlPointSequence'),
array(0x300A,0x02D2,'DS','1','ControlPointRelativePosition'),
array(0x300A,0x02D4,'DS','3','ControlPointDPosition'),
array(0x300A,0x02D6,'DS','1','CumulativeTimeWeight'),
array(0x300C,0x0000,'UL','1','RTRelationshipGroupLength'),
array(0x300C,0x0002,'SQ','1','ReferencedRTPlanSequence'),
array(0x300C,0x0004,'SQ','1','ReferencedBeamSequence'),
array(0x300C,0x0006,'IS','1','ReferencedBeamNumber'),
array(0x300C,0x0007,'IS','1','ReferencedReferenceImageNumber'),
array(0x300C,0x0008,'DS','1','StartCumulativeMetersetWeight'),
array(0x300C,0x0009,'DS','1','EndCumulativeMetersetWeight'),
array(0x300C,0x000A,'SQ','1','ReferencedBrachyApplicationSetupSequence'),
array(0x300C,0x000C,'IS','1','ReferencedBrachyApplicationSetupNumber'),
array(0x300C,0x000E,'IS','1','ReferencedSourceNumber'),
array(0x300C,0x0020,'SQ','1','ReferencedFractionGroupSequence'),
array(0x300C,0x0022,'IS','1','ReferencedFractionGroupNumber'),
array(0x300C,0x0040,'SQ','1','ReferencedVerificationImageSequence'),
array(0x300C,0x0042,'SQ','1','ReferencedReferenceImageSequence'),
array(0x300C,0x0050,'SQ','1','ReferencedDoseReferenceSequence'),
array(0x300C,0x0051,'IS','1','ReferencedDoseReferenceNumber'),
array(0x300C,0x0055,'SQ','1','BrachyReferencedDoseReferenceSequence'),
array(0x300C,0x0060,'SQ','1','ReferencedStructureSetSequence'),
array(0x300C,0x006A,'IS','1','ReferencedPatientSetupNumber'),
array(0x300C,0x0080,'SQ','1','ReferencedDoseSequence'),
array(0x300C,0x00A0,'IS','1','ReferencedToleranceTableNumber'),
array(0x300C,0x00B0,'SQ','1','ReferencedBolusSequence'),
array(0x300C,0x00C0,'IS','1','ReferencedWedgeNumber'),
array(0x300C,0x00D0,'IS','1','ReferencedCompensatorNumber'),
array(0x300C,0x00E0,'IS','1','ReferencedBlockNumber'),
array(0x300C,0x00F0,'IS','1','ReferencedControlPointIndex'),
array(0x300E,0x0000,'UL','1','RTApprovalGroupLength'),
array(0x300E,0x0002,'CS','1','ApprovalStatus'),
array(0x300E,0x0004,'DA','1','ReviewDate'),
array(0x300E,0x0005,'TM','1','ReviewTime'),
array(0x300E,0x0008,'PN','1','ReviewerName'),
array(0x4000,0x0000,'UL','1','TextGroupLength'),
array(0x4000,0x0010,'LT','1-n','TextArbitrary'),
array(0x4000,0x4000,'LT','1-n','TextComments'),
array(0x4008,0x0000,'UL','1','ResultsGroupLength'),
array(0x4008,0x0040,'SH','1','ResultsID'),
array(0x4008,0x0042,'LO','1','ResultsIDIssuer'),
array(0x4008,0x0050,'SQ','1','ReferencedInterpretationSequence'),
array(0x4008,0x0100,'DA','1','InterpretationRecordedDate'),
array(0x4008,0x0101,'TM','1','InterpretationRecordedTime'),
array(0x4008,0x0102,'PN','1','InterpretationRecorder'),
array(0x4008,0x0103,'LO','1','ReferenceToRecordedSound'),
array(0x4008,0x0108,'DA','1','InterpretationTranscriptionDate'),
array(0x4008,0x0109,'TM','1','InterpretationTranscriptionTime'),
array(0x4008,0x010A,'PN','1','InterpretationTranscriber'),
array(0x4008,0x010B,'ST','1','InterpretationText'),
array(0x4008,0x010C,'PN','1','InterpretationAuthor'),
array(0x4008,0x0111,'SQ','1','InterpretationApproverSequence'),
array(0x4008,0x0112,'DA','1','InterpretationApprovalDate'),
array(0x4008,0x0113,'TM','1','InterpretationApprovalTime'),
array(0x4008,0x0114,'PN','1','PhysicianApprovingInterpretation'),
array(0x4008,0x0115,'LT','1','InterpretationDiagnosisDescription'),
array(0x4008,0x0117,'SQ','1','DiagnosisCodeSequence'),
array(0x4008,0x0118,'SQ','1','ResultsDistributionListSequence'),
array(0x4008,0x0119,'PN','1','DistributionName'),
array(0x4008,0x011A,'LO','1','DistributionAddress'),
array(0x4008,0x0200,'SH','1','InterpretationID'),
array(0x4008,0x0202,'LO','1','InterpretationIDIssuer'),
array(0x4008,0x0210,'CS','1','InterpretationTypeID'),
array(0x4008,0x0212,'CS','1','InterpretationStatusID'),
array(0x4008,0x0300,'ST','1','Impressions'),
array(0x4008,0x4000,'ST','1','ResultsComments'),
array(0x5000,0x0000,'UL','1','CurveGroupLength'),
array(0x5000,0x0005,'US','1','CurveDimensions'),
array(0x5000,0x0010,'US','1','NumberOfPoints'),
array(0x5000,0x0020,'CS','1','TypeOfData'),
array(0x5000,0x0022,'LO','1','CurveDescription'),
array(0x5000,0x0030,'SH','1-n','AxisUnits'),
array(0x5000,0x0040,'SH','1-n','AxisLabels'),
array(0x5000,0x0103,'US','1','DataValueRepresentation'),
array(0x5000,0x0104,'US','1-n','MinimumCoordinateValue'),
array(0x5000,0x0105,'US','1-n','MaximumCoordinateValue'),
array(0x5000,0x0106,'SH','1-n','CurveRange'),
array(0x5000,0x0110,'US','1','CurveDataDescriptor'),
array(0x5000,0x0112,'US','1','CoordinateStartValue'),
array(0x5000,0x0114,'US','1','CoordinateStepValue'),
array(0x5000,0x2000,'US','1','AudioType'),
array(0x5000,0x2002,'US','1','AudioSampleFormat'),
array(0x5000,0x2004,'US','1','NumberOfChannels'),
array(0x5000,0x2006,'UL','1','NumberOfSamples'),
array(0x5000,0x2008,'UL','1','SampleRate'),
array(0x5000,0x200A,'UL','1','TotalTime'),
array(0x5000,0x200C,'OX','1','AudioSampleData'),
array(0x5000,0x200E,'LT','1','AudioComments'),
array(0x5000,0x3000,'OX','1','CurveData'),
array(0x5400,0x0100,'SQ','1','WaveformSequence'),
array(0x5400,0x0110,'OW/OB','1','ChannelMinimumValue'),
array(0x5400,0x0112,'OW/OB','1','ChannelMaximumValue'),
array(0x5400,0x1004,'US','1','WaveformBitsAllocated'),
array(0x5400,0x1006,'CS','1','WaveformSampleInterpretation'),
array(0x5400,0x100A,'OW/OB','1','WaveformPaddingValue'),
array(0x5400,0x1010,'OW/OB','1','WaveformData'),
array(0x6000,0x0000,'UL','1','OverlayGroupLength'),
array(0x6000,0x0010,'US','1','OverlayRows'),
array(0x6000,0x0011,'US','1','OverlayColumns'),
array(0x6000,0x0012,'US','1','OverlayPlanes'),
array(0x6000,0x0015,'IS','1','OverlayNumberOfFrames'),
array(0x6000,0x0040,'CS','1','OverlayType'),
array(0x6000,0x0050,'SS','2','OverlayOrigin'),
array(0x6000,0x0051,'US','1','OverlayImageFrameOrigin'),
array(0x6000,0x0052,'US','1','OverlayPlaneOrigin'),
array(0x6000,0x0060,'CS','1','OverlayCompressionCode'),
array(0x6000,0x0061,'SH','1','OverlayCompressionOriginator'),
array(0x6000,0x0062,'SH','1','OverlayCompressionLabel'),
array(0x6000,0x0063,'SH','1','OverlayCompressionDescription'),
array(0x6000,0x0066,'AT','1-n','OverlayCompressionStepPointers'),
array(0x6000,0x0068,'US','1','OverlayRepeatInterval'),
array(0x6000,0x0069,'US','1','OverlayBitsGrouped'),
array(0x6000,0x0100,'US','1','OverlayBitsAllocated'),
array(0x6000,0x0102,'US','1','OverlayBitPosition'),
array(0x6000,0x0110,'CS','1','OverlayFormat'),
array(0x6000,0x0200,'US','1','OverlayLocation'),
array(0x6000,0x0800,'CS','1-n','OverlayCodeLabel'),
array(0x6000,0x0802,'US','1','OverlayNumberOfTables'),
array(0x6000,0x0803,'AT','1-n','OverlayCodeTableLocation'),
array(0x6000,0x0804,'US','1','OverlayBitsForCodeWord'),
array(0x6000,0x1100,'US','1','OverlayDescriptorGray'),
array(0x6000,0x1101,'US','1','OverlayDescriptorRed'),
array(0x6000,0x1102,'US','1','OverlayDescriptorGreen'),
array(0x6000,0x1103,'US','1','OverlayDescriptorBlue'),
array(0x6000,0x1200,'US','1-n','OverlayGray'),
array(0x6000,0x1201,'US','1-n','OverlayRed'),
array(0x6000,0x1202,'US','1-n','OverlayGreen'),
array(0x6000,0x1203,'US','1-n','OverlayBlue'),
array(0x6000,0x1301,'IS','1','ROIArea'),
array(0x6000,0x1302,'DS','1','ROIMean'),
array(0x6000,0x1303,'DS','1','ROIStandardDeviation'),
array(0x6000,0x3000,'OW','1','OverlayData'),
array(0x6000,0x4000,'LT','1-n','OverlayComments'),
array(0x7F00,0x0000,'UL','1','VariablePixelDataGroupLength'),
array(0x7F00,0x0010,'OX','1','VariablePixelData'),
array(0x7F00,0x0011,'AT','1','VariableNextDataGroup'),
array(0x7F00,0x0020,'OW','1-n','VariableCoefficientsSDVN'),
array(0x7F00,0x0030,'OW','1-n','VariableCoefficientsSDHN'),
array(0x7F00,0x0040,'OW','1-n','VariableCoefficientsSDDN'),
array(0x7FE0,0x0000,'UL','1','PixelDataGroupLength'),
array(0x7FE0,0x0010,'OX','1','PixelData'),
array(0x7FE0,0x0020,'OW','1-n','CoefficientsSDVN'),
array(0x7FE0,0x0030,'OW','1-n','CoefficientsSDHN'),
array(0x7FE0,0x0040,'OW','1-n','CoefficientsSDDN'),
array(0xFFFC,0xFFFC,'OB','1','DataSetTrailingPadding'),
array(0xFFFE,0xE000,'NONE','1','Item'),
array(0xFFFE,0xE00D,'NONE','1','ItemDelimitationItem'),
array(0xFFFE,0xE0DD,'NONE','1','SequenceDelimitationItem')
                             );

    /**
    * Array of DICOM elements
    * @var array
    */
    var $_elements;

    /**
    * Initialize dictionary only once.
    * Read the contents of the DICOM dictionary into a hash by group and element.
    * dicom_private is read after dicom_fields, so overrides common fields.
    *
    * @access public
    */
    function File_DICOM()
    {
        foreach ($this->dicom_fields as $field) {
            list($group, $elem, $code, $numa, $name) = $field;
            $this->dict[$group][$elem] = array($code, $name);
        }
    }

    /**
    * Fill in hash with header members from given file.
    *
    * @access public
    * @param string $infile The DICOM file to parse
    * @return mixed true on success, PEAR_Error on failure
    */
    function parse($infile)
    {
        $this->current_file = $infile;
        $fh = fopen($infile, "rb");
        if (!$fh) {
            return $this->raiseError("Could not open file $infile for reading");
        }
        $stat = fstat($fh);
        $this->_file_length = $stat[7];
  
        // Test for NEMA or DICOM file.  
        // If DICM, store initial preamble and leave file ptr at 0x84.
        $this->_preamble_buff = fread($fh, 0x80);
        $buff = fread($fh, 4);
        $this->is_dicm = ($buff == 'DICM');
        if (!$this->is_dicm) {
            fseek($fh, 0);
        }
  
        while (ftell($fh) < $this->_file_length) {
            $element = new File_DICOM_Element($fh, $this->dict);
            $gp = $element->group;
            $el = $element->element;
            $this->_elements[$gp][$el] = $element;
        }
        fclose($fh);
        return true;
    }

    /**
    * Write current contents to a file.
    *
    * @access public
    * @param string $outfile The name of the file to write. If not given it assumes
    *                        the name of the file parsed. If no file was parsed 
    *                        and no name is given returns a PEAR_Error
    * @return mixed true on success, PEAR_Error on failure
    */
    function write($outfile = '')
    {
        if ($outfile == '') {
            if (isset($this->current_file)) {
                $outfile = $this->current_file;
            } else {
                return $this->raiseError("File name not given (and no file currently open)");
            }
        }
        $fh = fopen($outfile, "wb");
        if (!$fh) {
            return $this->raiseError("Could not open file $outfile for writing");
        }
  
        // Writing file from scratch will always fail for now
        if (!isset($this->_preamble_buff)) {
            return $this->raiseError("Cannot write DICOM file from scratch");
        }
        // Don't store initial preamble and DICM word. Working with NEMA.
        //fwrite($fh, $this->_preamble_buff);
        //fwrite($fh, 'DICM');
        /*$buff = fread($fh, 4);
        $this->is_dicm = ($buff == 'DICM');
        if (!$this->is_dicm) {
            fseek($fh, 0);
        }*/
 
        // There are not that much groups/elements. Using foreach()
        foreach (array_keys($this->_elements) as $group) {
            foreach (array_keys($this->_elements[$group]) as $element) {
                fwrite($fh, pack('v', $group));
                fwrite($fh, pack('v', $element));
                $code = $this->_elements[$group][$element]->code;
                // Preserve the VR type from the file parsed
                if (($this->_elements[$group][$element]->vr_type == FILE_DICOM_VR_TYPE_EXPLICIT_32_BITS) or ($this->_elements[$group][$element]->vr_type == FILE_DICOM_VR_TYPE_EXPLICIT_16_BITS)) {
                    fwrite($fh, pack('CC', $code{0}, $code{1}));
                    // This is an OB, OW, SQ, UN or UT: 32 bit VL field.
                    if ($this->_elements[$group][$element]->vr_type == FILE_DICOM_VR_TYPE_EXPLICIT_32_BITS) {
                        fwrite($fh, pack('V', $this->_elements[$group][$element]->length));
                    } else { // not using fixed length from VR!!!
                        fwrite($fh, pack('v', $this->_elements[$group][$element]->length));
                    }
                } elseif ($this->_elements[$group][$element]->vr_type == FILE_DICOM_VR_TYPE_IMPLICIT) {
                    fwrite($fh, pack('V', $this->_elements[$group][$element]->length));
                }
                switch ($code) {
                    // Decode unsigned longs and shorts.
                    case 'UL':
                        fwrite($fh, pack('V', $this->_elements[$group][$element]->value));
                        break;
                    case 'US':
                        fwrite($fh, pack('v', $this->_elements[$group][$element]->value));
                        break;
                    // Floats: Single and double precision.
                    case 'FL':
                        fwrite($fh, pack('f', $this->_elements[$group][$element]->value));
                        break;
                    case 'FD':
                        fwrite($fh, pack('d', $this->_elements[$group][$element]->value));
                        break;
                    // Binary data. Only save position. Is this right? 
                    case 'OW':
                    case 'OB':
                    case 'OX':
                        // Binary data. Value only contains position on the parsed file.
                        // Will fail when file name for writing is the same as for parsing.
                        $fh2 = fopen($this->current_file, "rb");
                        if (!$fh2) {
                            return $this->raiseError("Could not open file {$this->current_file} for reading");
                        }
                        fseek($fh2, $this->_elements[$group][$element]->value);
                        fwrite($fh, fread($fh2, $this->_elements[$group][$element]->length));
                        fclose($fh2);
                        break;
                    default:
                        fwrite($fh, $this->_elements[$group][$element]->value, $this->_elements[$group][$element]->length);
                        break;
                }
            }
        }
        fclose($fh);
        return true;
    }

    /**
    * Gets the value for a DICOM element
    *
    * @access public
    * @param integer $gp The group the DICOM element belongs to
    * @param integer $el The identifier for the DICOM element (unique inside a group)
    * @return mixed The value for the DICOM element on success, PEAR_Error on failure 
    */
    function getValue($gp, $el)
    {
        if (isset($this->_elements[$gp][$el])) {
            $elem = $this->_elements[$gp][$el];
        } else {
            return $this->raiseError("Element ($gp,$el) not found");
        }
        $value = $elem->getValue();
        return $value;
    }

    /**
    * Sets the value for a DICOM element
    * Only works with strings now.
    *
    * @access public
    * @param integer $gp The group the DICOM element belongs to
    * @param integer $el The identifier for the DICOM element (unique inside a group)
    */
    function setValue($gp, $el, $value)
    {
        $this->_elements[$gp][$el]->value = $value;
        $this->_elements[$gp][$el]->length = strlen($value);
    }

    /**
    * Dumps the contents of the image inside the DICOM file 
    * (element 0x0010 from group 0x7FE0) to a PGM (Portable Gray Map) file.
    * Use with Caution!!. For a 8.5MB DICOM file on a P4 it takes 28 
    * seconds to dump it's image.
    *
    * @access public
    * @param string  $filename The file where to save the image
    * @return mixed true on success, PEAR_Error on failure.
    */
    function dumpImage($filename)
    {
        $length = $this->_elements[0x7FE0][0x0010]->length;
        $rows = $this->getValue(0x0028,0x0010);
        $cols = $this->getValue(0x0028,0x0011);
        $fh = fopen($filename, "wb");
        if (!$fh) {
            return $this->raiseError("Could not open file $filename for writing");
        }
        // magick word
        fwrite($fh, "P5\n");
        // comment
        fwrite($fh, "# file generated by PEAR::File_DICOM on ".strftime("%Y-%m-%d", time())." \n");
        fwrite($fh, "# do not use for diagnosing purposes\n");
        fwrite($fh, "$cols $rows\n");
        // always 255 grays
        fwrite($fh, "255\n");
        $pos = $this->getValue(0x7FE0,0x0010);
        $fh_in = fopen($this->current_file, "rb");
        if (!$fh_in) {
            return $this->raiseError("Could not open file {$this->current_file} for reading");
        }
        fseek($fh_in, $pos);
        $block_size = 4096;
        $blocks = ceil($length / $block_size);
        for ($i = 0; $i < $blocks; $i++) {
            if ($i == $blocks - 1) { // last block
                $chunk_length = ($length % $block_size) ? ($length % $block_size) : $block_size;
            } else {
                $chunk_length = $block_size;
            }
            $chunk = fread($fh_in, $chunk_length);
            $pgm = '';
            $half_chunk_length = $chunk_length/2;
            $rr = unpack("v$half_chunk_length", $chunk);
            for ($j = 1; $j <= $half_chunk_length; $j++) {
                $pgm .= pack('C', $rr[$j] >> 4);
            }
            fwrite($fh, $pgm);
        }
        fclose($fh_in);
        fclose($fh);
        return true;
    }
}
?>
